@@ -109,17 +109,13 @@ dump_to_json(VALUE obj, Out out) {
     const char		*s;
     int			len;
 
-    if (RB_UNLIKELY(Yes == out->opts->trace)) {
-	oj_trace("to_json", obj, __FILE__, __LINE__, 0, TraceRubyIn);
-    }
+    TRACE(out->opts->trace, "to_json", obj, 0, TraceRubyIn);
     if (0 == rb_obj_method_arity(obj, oj_to_json_id)) {
 	rs = rb_funcall(obj, oj_to_json_id, 0);
     } else {
 	rs = rb_funcall2(obj, oj_to_json_id, out->argc, out->argv);
     }
-    if (RB_UNLIKELY(Yes == out->opts->trace)) {
-	oj_trace("to_json", obj, __FILE__, __LINE__, 0, TraceRubyOut);
-    }
+    TRACE(out->opts->trace, "to_json", obj, 0, TraceRubyOut);
 
     s = RSTRING_PTR(rs);
     len = (int)RSTRING_LEN(rs);
@@ -299,7 +295,7 @@ datetime_alt(VALUE obj, int depth, Out out) {
     attrs[3].value = rb_funcall(obj, hour_id, 0);
     attrs[4].value = rb_funcall(obj, min_id, 0);
     attrs[5].value = rb_funcall(obj, sec_id, 0);
-    attrs[6].value = rb_funcall(rb_funcall(obj, offset_id, 0), oj_to_s_id, 0);
+    attrs[6].value = oj_safe_string_convert(rb_funcall(obj, offset_id, 0));
     attrs[7].value = rb_funcall(obj, start_id, 0);
 
     oj_code_attrs(obj, attrs, depth, out, true);
@@ -606,7 +602,7 @@ dump_float(VALUE obj, int depth, Out out, bool as_ok) {
     } else if (oj_rails_float_opt) {
 	cnt = oj_dump_float_printf(buf, sizeof(buf), obj, d, "%0.16g");
     } else {
-	volatile VALUE	rstr = rb_funcall(obj, oj_to_s_id, 0);
+	volatile VALUE	rstr = oj_safe_string_convert(obj);
 
 	strcpy(buf, RSTRING_PTR(rstr));
 	cnt = (int)RSTRING_LEN(rstr);
@@ -648,7 +644,7 @@ hash_cb(VALUE key, VALUE value, VALUE ov) {
 	break;
     default:
 	/*rb_raise(rb_eTypeError, "In :compat mode all Hash keys must be Strings or Symbols, not %s.\n", rb_class2name(rb_obj_class(key)));*/
-	oj_dump_str(rb_funcall(key, oj_to_s_id, 0), 0, out, false);
+	oj_dump_str(oj_safe_string_convert(key), 0, out, false);
 	break;
     }
     if (!out->opts->dump_opts.use) {
@@ -833,7 +829,7 @@ dump_bignum(VALUE obj, int depth, Out out, bool as_ok) {
     if (use_bignum_alt) {
 	rs = rb_big2str(obj, 10);
     } else {
-	rs = rb_funcall(obj, oj_to_s_id, 0);
+	rs = oj_safe_string_convert(obj);
     }
     rb_check_type(rs, T_STRING);
     cnt = (int)RSTRING_LEN(rs);
@@ -893,9 +889,7 @@ void
 oj_dump_compat_val(VALUE obj, int depth, Out out, bool as_ok) {
     int	type = rb_type(obj);
 
-    if (RB_UNLIKELY(Yes == out->opts->trace)) {
-	oj_trace("dump", obj, __FILE__, __LINE__, depth, TraceIn);
-    }
+    TRACE(out->opts->trace, "dump", obj, depth, TraceIn);
     if (out->opts->dump_opts.max_depth <= depth) {
 	// When JSON.dump is called then an ArgumentError is expected and the
 	// limit is the depth inclusive. If JSON.generate is called then a
@@ -918,14 +912,10 @@ oj_dump_compat_val(VALUE obj, int depth, Out out, bool as_ok) {
 
 	if (NULL != f) {
 	    f(obj, depth, out, as_ok);
-	    if (RB_UNLIKELY(Yes == out->opts->trace)) {
-		oj_trace("dump", obj, __FILE__, __LINE__, depth, TraceOut);
-	    }
+	    TRACE(out->opts->trace, "dump", obj, depth, TraceOut);
 	    return;
 	}
     }
     oj_dump_nil(Qnil, depth, out, false);
-    if (RB_UNLIKELY(Yes == out->opts->trace)) {
-	oj_trace("dump", Qnil, __FILE__, __LINE__, depth, TraceOut);
-    }
+    TRACE(out->opts->trace, "dump", Qnil, depth, TraceOut);
 }
