@@ -1,6 +1,7 @@
 // Copyright (c) 2012, 2017 Peter Ohler. All rights reserved.
 // Licensed under the MIT License. See LICENSE file in the project root for license details.
 
+#include "mem.h"
 #include "dump.h"
 #include "odd.h"
 #include "trace.h"
@@ -24,18 +25,13 @@ static void dump_data(VALUE obj, int depth, Out out, bool as_ok) {
 
     if (rb_cTime == clas) {
         assure_size(out, 6);
-        *out->cur++ = '{';
-        *out->cur++ = '"';
-        *out->cur++ = '^';
-        *out->cur++ = 't';
-        *out->cur++ = '"';
-        *out->cur++ = ':';
+        APPEND_CHARS(out->cur, "{\"^t\":", 6);
         dump_time(obj, out);
         *out->cur++ = '}';
         *out->cur   = '\0';
     } else {
         if (oj_bigdecimal_class == clas) {
-            volatile VALUE rstr = rb_funcall(obj, oj_to_s_id, 0);
+            volatile VALUE rstr = oj_safe_string_convert(obj);
             const char *   str  = RSTRING_PTR(rstr);
             int            len  = (int)RSTRING_LEN(rstr);
 
@@ -64,7 +60,7 @@ static void dump_obj(VALUE obj, int depth, Out out, bool as_ok) {
     VALUE clas = rb_obj_class(obj);
 
     if (oj_bigdecimal_class == clas) {
-        volatile VALUE rstr = rb_funcall(obj, oj_to_s_id, 0);
+        volatile VALUE rstr = oj_safe_string_convert(obj);
         const char *   str  = RSTRING_PTR(rstr);
         int            len  = (int)RSTRING_LEN(rstr);
 
@@ -91,12 +87,7 @@ static void dump_class(VALUE obj, int depth, Out out, bool as_ok) {
     size_t      len = strlen(s);
 
     assure_size(out, 6);
-    *out->cur++ = '{';
-    *out->cur++ = '"';
-    *out->cur++ = '^';
-    *out->cur++ = 'c';
-    *out->cur++ = '"';
-    *out->cur++ = ':';
+    APPEND_CHARS(out->cur, "{\"^c\":", 6);
     oj_dump_cstr(s, len, 0, 0, out);
     *out->cur++ = '}';
     *out->cur   = '\0';
@@ -120,9 +111,7 @@ static void dump_array_class(VALUE a, VALUE clas, int depth, Out out) {
     if (0 < id) {
         assure_size(out, d2 * out->indent + 16);
         fill_indent(out, d2);
-        *out->cur++ = '"';
-        *out->cur++ = '^';
-        *out->cur++ = 'i';
+        APPEND_CHARS(out->cur, "\"^i", 3);
         dump_ulong(id, out);
         *out->cur++ = '"';
     }
@@ -139,25 +128,23 @@ static void dump_array_class(VALUE a, VALUE clas, int depth, Out out) {
         } else {
             size = d2 * out->indent + 2;
         }
+        assure_size(out, size * cnt);
         cnt--;
         for (i = 0; i <= cnt; i++) {
-            assure_size(out, size);
             if (out->opts->dump_opts.use) {
                 if (0 < out->opts->dump_opts.array_size) {
-                    strcpy(out->cur, out->opts->dump_opts.array_nl);
-                    out->cur += out->opts->dump_opts.array_size;
+                    APPEND_CHARS(out->cur, out->opts->dump_opts.array_nl, out->opts->dump_opts.array_size);
                 }
                 if (0 < out->opts->dump_opts.indent_size) {
                     int i;
                     for (i = d2; 0 < i; i--) {
-                        strcpy(out->cur, out->opts->dump_opts.indent_str);
-                        out->cur += out->opts->dump_opts.indent_size;
+                        APPEND_CHARS(out->cur, out->opts->dump_opts.indent_str, out->opts->dump_opts.indent_size);
                     }
                 }
             } else {
                 fill_indent(out, d2);
             }
-            oj_dump_obj_val(rb_ary_entry(a, i), d2, out);
+            oj_dump_obj_val(RARRAY_AREF(a, i), d2, out);
             if (i < cnt) {
                 *out->cur++ = ',';
             }
@@ -168,15 +155,13 @@ static void dump_array_class(VALUE a, VALUE clas, int depth, Out out) {
             // printf("*** d2: %u  indent: %u '%s'\n", d2, out->opts->dump_opts->indent_size,
             // out->opts->dump_opts->indent);
             if (0 < out->opts->dump_opts.array_size) {
-                strcpy(out->cur, out->opts->dump_opts.array_nl);
-                out->cur += out->opts->dump_opts.array_size;
+                APPEND_CHARS(out->cur, out->opts->dump_opts.array_nl, out->opts->dump_opts.array_size);
             }
             if (0 < out->opts->dump_opts.indent_size) {
                 int i;
 
                 for (i = depth; 0 < i; i--) {
-                    strcpy(out->cur, out->opts->dump_opts.indent_str);
-                    out->cur += out->opts->dump_opts.indent_size;
+                    APPEND_CHARS(out->cur, out->opts->dump_opts.indent_str, out->opts->dump_opts.indent_size);
                 }
             }
         } else {
@@ -218,7 +203,7 @@ static int hash_cb(VALUE key, VALUE value, VALUE ov) {
     int  depth = out->depth;
     long size  = depth * out->indent + 1;
 
-    if (oj_dump_ignore(out->opts, value)) {
+    if (dump_ignore(out->opts, value)) {
         return ST_CONTINUE;
     }
     if (out->omit_nil && Qnil == value) {
@@ -226,47 +211,50 @@ static int hash_cb(VALUE key, VALUE value, VALUE ov) {
     }
     assure_size(out, size);
     fill_indent(out, depth);
-    if (rb_type(key) == T_STRING) {
+    switch (rb_type(key)) {
+    case T_STRING:
         dump_str_class(key, Qundef, depth, out);
         *out->cur++ = ':';
         oj_dump_obj_val(value, depth, out);
-    } else if (rb_type(key) == T_SYMBOL) {
+        break;
+
+    case T_SYMBOL:
         dump_sym(key, 0, out, false);
         *out->cur++ = ':';
         oj_dump_obj_val(value, depth, out);
-    } else {
-        int     d2 = depth + 1;
-        long    s2 = size + out->indent + 1;
-        int     i;
-        int     started = 0;
-        uint8_t b;
+        break;
 
-        assure_size(out, s2 + 15);
-        *out->cur++ = '"';
-        *out->cur++ = '^';
-        *out->cur++ = '#';
-        out->hash_cnt++;
-        for (i = 28; 0 <= i; i -= 4) {
-            b = (uint8_t)((out->hash_cnt >> i) & 0x0000000F);
-            if ('\0' != b) {
-                started = 1;
-            }
-            if (started) {
-                *out->cur++ = hex_chars[b];
+    default:
+        {
+            int     d2 = depth + 1;
+            long    s2 = size + out->indent + 1;
+            int     i;
+            int     started = 0;
+            uint8_t b;
+
+            assure_size(out, s2 + 15);
+            APPEND_CHARS(out->cur, "\"^#", 3);
+            out->hash_cnt++;
+            for (i = 28; 0 <= i; i -= 4) {
+                b = (uint8_t)((out->hash_cnt >> i) & 0x0000000F);
+                if ('\0' != b) {
+                    started = 1;
+                }
+                if (started) {
+                    *out->cur++ = hex_chars[b];
+                }
             }
+            APPEND_CHARS(out->cur, "\":[", 3);
+            fill_indent(out, d2);
+            oj_dump_obj_val(key, d2, out);
+            assure_size(out, s2);
+            *out->cur++ = ',';
+            fill_indent(out, d2);
+            oj_dump_obj_val(value, d2, out);
+            assure_size(out, size);
+            fill_indent(out, depth);
+            *out->cur++ = ']';
         }
-        *out->cur++ = '"';
-        *out->cur++ = ':';
-        *out->cur++ = '[';
-        fill_indent(out, d2);
-        oj_dump_obj_val(key, d2, out);
-        assure_size(out, s2);
-        *out->cur++ = ',';
-        fill_indent(out, d2);
-        oj_dump_obj_val(value, d2, out);
-        assure_size(out, size);
-        fill_indent(out, depth);
-        *out->cur++ = ']';
     }
     out->depth  = depth;
     *out->cur++ = ',';
@@ -286,8 +274,7 @@ static void dump_hash_class(VALUE obj, VALUE clas, int depth, Out out) {
     size = depth * out->indent + 2;
     assure_size(out, 2);
     if (0 == cnt) {
-        *out->cur++ = '{';
-        *out->cur++ = '}';
+        APPEND_CHARS(out->cur, "{}", 2);
     } else {
         long id = oj_check_circular(obj, out);
 
@@ -298,11 +285,7 @@ static void dump_hash_class(VALUE obj, VALUE clas, int depth, Out out) {
         if (0 < id) {
             assure_size(out, size + 16);
             fill_indent(out, depth + 1);
-            *out->cur++ = '"';
-            *out->cur++ = '^';
-            *out->cur++ = 'i';
-            *out->cur++ = '"';
-            *out->cur++ = ':';
+            APPEND_CHARS(out->cur, "\"^i\":", 5);
             dump_ulong(id, out);
             *out->cur++ = ',';
         }
@@ -318,15 +301,13 @@ static void dump_hash_class(VALUE obj, VALUE clas, int depth, Out out) {
             size = depth * out->opts->dump_opts.indent_size + out->opts->dump_opts.hash_size + 1;
             assure_size(out, size);
             if (0 < out->opts->dump_opts.hash_size) {
-                strcpy(out->cur, out->opts->dump_opts.hash_nl);
-                out->cur += out->opts->dump_opts.hash_size;
+                APPEND_CHARS(out->cur, out->opts->dump_opts.hash_nl, out->opts->dump_opts.hash_size);
             }
             if (0 < out->opts->dump_opts.indent_size) {
                 int i;
 
                 for (i = depth; 0 < i; i--) {
-                    strcpy(out->cur, out->opts->dump_opts.indent_str);
-                    out->cur += out->opts->dump_opts.indent_size;
+                    APPEND_CHARS(out->cur, out->opts->dump_opts.indent_str, out->opts->dump_opts.indent_size);
                 }
             }
         }
@@ -335,14 +316,13 @@ static void dump_hash_class(VALUE obj, VALUE clas, int depth, Out out) {
     *out->cur = '\0';
 }
 
-#ifdef HAVE_RB_IVAR_FOREACH
 static int dump_attr_cb(ID key, VALUE value, VALUE ov) {
     Out         out   = (Out)ov;
     int         depth = out->depth;
     size_t      size  = depth * out->indent + 1;
     const char *attr  = rb_id2name(key);
 
-    if (oj_dump_ignore(out->opts, value)) {
+    if (dump_ignore(out->opts, value)) {
         return ST_CONTINUE;
     }
     if (out->omit_nil && Qnil == value) {
@@ -378,7 +358,6 @@ static int dump_attr_cb(ID key, VALUE value, VALUE ov) {
 
     return ST_CONTINUE;
 }
-#endif
 
 static void dump_hash(VALUE obj, int depth, Out out, bool as_ok) {
     dump_hash_class(obj, rb_obj_class(obj), depth, out);
@@ -401,11 +380,7 @@ static void dump_odd(VALUE obj, Odd odd, VALUE clas, int depth, Out out) {
         size = d2 * out->indent + clen + 10;
         assure_size(out, size);
         fill_indent(out, d2);
-        *out->cur++ = '"';
-        *out->cur++ = '^';
-        *out->cur++ = 'O';
-        *out->cur++ = '"';
-        *out->cur++ = ':';
+        APPEND_CHARS(out->cur, "\"^O\":", 5);
         oj_dump_cstr(class_name, clen, 0, 0, out);
         *out->cur++ = ',';
     }
@@ -423,12 +398,9 @@ static void dump_odd(VALUE obj, Odd odd, VALUE clas, int depth, Out out) {
             assure_size(out, size);
             fill_indent(out, d2);
             *out->cur++ = '"';
-            memcpy(out->cur, name, nlen);
-            out->cur += nlen;
-            *out->cur++ = '"';
-            *out->cur++ = ':';
-            memcpy(out->cur, s, len);
-            out->cur += len;
+            APPEND_CHARS(out->cur, name, nlen);
+            APPEND_CHARS(out->cur, "\":", 2);
+            APPEND_CHARS(out->cur, s, len);
             *out->cur = '\0';
         }
     } else {
@@ -439,7 +411,7 @@ static void dump_odd(VALUE obj, Odd odd, VALUE clas, int depth, Out out) {
             assure_size(out, size);
             name = rb_id2name(*idp);
             nlen = strlen(name);
-            if (0 != *fp) {
+            if (NULL != *fp) {
                 v = (*fp)(obj);
             } else if (0 == strchr(name, '.')) {
                 v = rb_funcall(obj, *idp, 0);
@@ -451,7 +423,7 @@ static void dump_odd(VALUE obj, Odd odd, VALUE clas, int depth, Out out) {
                 ID    i;
 
                 if (sizeof(nbuf) <= nlen) {
-                    if (NULL == (n2 = strdup(name))) {
+                    if (NULL == (n2 = OJ_STRDUP(name))) {
                         rb_raise(rb_eNoMemError, "for attribute name.");
                     }
                 } else {
@@ -468,7 +440,7 @@ static void dump_odd(VALUE obj, Odd odd, VALUE clas, int depth, Out out) {
                 i = rb_intern(n);
                 v = rb_funcall(v, i, 0);
                 if (nbuf != n2) {
-                    free(n2);
+                    OJ_FREE(n2);
                 }
             }
             fill_indent(out, d2);
@@ -502,22 +474,14 @@ static void dump_obj_attrs(VALUE obj, VALUE clas, slot_t id, int depth, Out out)
 
         assure_size(out, d2 * out->indent + clen + 10);
         fill_indent(out, d2);
-        *out->cur++ = '"';
-        *out->cur++ = '^';
-        *out->cur++ = 'o';
-        *out->cur++ = '"';
-        *out->cur++ = ':';
+        APPEND_CHARS(out->cur, "\"^o\":", 5);
         oj_dump_cstr(class_name, clen, 0, 0, out);
     }
     if (0 < id) {
         assure_size(out, d2 * out->indent + 16);
         *out->cur++ = ',';
         fill_indent(out, d2);
-        *out->cur++ = '"';
-        *out->cur++ = '^';
-        *out->cur++ = 'i';
-        *out->cur++ = '"';
-        *out->cur++ = ':';
+        APPEND_CHARS(out->cur, "\"^i\":", 5);
         dump_ulong(id, out);
     }
     switch (type) {
@@ -525,57 +489,28 @@ static void dump_obj_attrs(VALUE obj, VALUE clas, slot_t id, int depth, Out out)
         assure_size(out, d2 * out->indent + 14);
         *out->cur++ = ',';
         fill_indent(out, d2);
-        *out->cur++ = '"';
-        *out->cur++ = 's';
-        *out->cur++ = 'e';
-        *out->cur++ = 'l';
-        *out->cur++ = 'f';
-        *out->cur++ = '"';
-        *out->cur++ = ':';
+        APPEND_CHARS(out->cur, "\"self\":", 7);
         oj_dump_cstr(RSTRING_PTR(obj), (int)RSTRING_LEN(obj), 0, 0, out);
         break;
     case T_ARRAY:
         assure_size(out, d2 * out->indent + 14);
         *out->cur++ = ',';
         fill_indent(out, d2);
-        *out->cur++ = '"';
-        *out->cur++ = 's';
-        *out->cur++ = 'e';
-        *out->cur++ = 'l';
-        *out->cur++ = 'f';
-        *out->cur++ = '"';
-        *out->cur++ = ':';
+        APPEND_CHARS(out->cur, "\"self\":", 7);
         dump_array_class(obj, Qundef, depth + 1, out);
         break;
     case T_HASH:
         assure_size(out, d2 * out->indent + 14);
         *out->cur++ = ',';
         fill_indent(out, d2);
-        *out->cur++ = '"';
-        *out->cur++ = 's';
-        *out->cur++ = 'e';
-        *out->cur++ = 'l';
-        *out->cur++ = 'f';
-        *out->cur++ = '"';
-        *out->cur++ = ':';
+        APPEND_CHARS(out->cur, "\"self\":", 7);
         dump_hash_class(obj, Qundef, depth + 1, out);
         break;
     default: break;
     }
     {
-        int cnt;
-#ifdef HAVE_RB_IVAR_COUNT
-        cnt = (int)rb_ivar_count(obj);
-#else
-        volatile VALUE vars = rb_funcall2(obj, oj_instance_variables_id, 0, 0);
-        VALUE *        np   = RARRAY_PTR(vars);
-        ID             vid;
-        const char *   attr;
-        int            i;
-        int            first = 1;
+        int cnt = (int)rb_ivar_count(obj);
 
-        cnt  = (int)RARRAY_LEN(vars);
-#endif
         if (Qundef != clas && 0 < cnt) {
             *out->cur++ = ',';
         }
@@ -588,52 +523,10 @@ static void dump_obj_attrs(VALUE obj, VALUE clas, slot_t id, int depth, Out out)
             }
         }
         out->depth = depth + 1;
-#ifdef HAVE_RB_IVAR_FOREACH
         rb_ivar_foreach(obj, dump_attr_cb, (VALUE)out);
         if (',' == *(out->cur - 1)) {
             out->cur--;  // backup to overwrite last comma
         }
-#else
-        size = d2 * out->indent + 1;
-        for (i = cnt; 0 < i; i--, np++) {
-            VALUE value;
-
-            vid  = rb_to_id(*np);
-            attr = rb_id2name(vid);
-            if (Yes == out->opts->ignore_under && '@' == *attr && '_' == attr[1]) {
-                continue;
-            }
-            value = rb_ivar_get(obj, vid);
-
-            if (oj_dump_ignore(out->opts, value)) {
-                continue;
-            }
-            if (out->omit_nil && Qnil == value) {
-                continue;
-            }
-            if (first) {
-                first = 0;
-            } else {
-                *out->cur++ = ',';
-            }
-            assure_size(out, size);
-            fill_indent(out, d2);
-            if ('@' == *attr) {
-                attr++;
-                oj_dump_cstr(attr, strlen(attr), 0, 0, out);
-            } else {
-                char buf[32];
-
-                *buf = '~';
-                strncpy(buf + 1, attr, sizeof(buf) - 2);
-                buf[sizeof(buf) - 1] = '\0';
-                oj_dump_cstr(buf, strlen(attr) + 1, 0, 0, out);
-            }
-            *out->cur++ = ':';
-            oj_dump_obj_val(value, d2, out);
-            assure_size(out, 2);
-        }
-#endif
         if (rb_obj_is_kind_of(obj, rb_eException)) {
             volatile VALUE rv;
 
@@ -681,12 +574,7 @@ static void dump_struct(VALUE obj, int depth, Out out, bool as_ok) {
     assure_size(out, size);
     *out->cur++ = '{';
     fill_indent(out, d2);
-    *out->cur++ = '"';
-    *out->cur++ = '^';
-    *out->cur++ = 'u';
-    *out->cur++ = '"';
-    *out->cur++ = ':';
-    *out->cur++ = '[';
+    APPEND_CHARS(out->cur, "\"^u\":[", 6);
     if ('#' == *class_name) {
         VALUE       ma = rb_struct_s_members(clas);
         const char *name;
@@ -694,7 +582,7 @@ static void dump_struct(VALUE obj, int depth, Out out, bool as_ok) {
 
         *out->cur++ = '[';
         for (i = 0; i < cnt; i++) {
-            volatile VALUE s = rb_sym2str(rb_ary_entry(ma, i));
+            volatile VALUE s = rb_sym2str(RARRAY_AREF(ma, i));
 
             name = RSTRING_PTR(s);
             len  = (int)RSTRING_LEN(s);
@@ -704,16 +592,14 @@ static void dump_struct(VALUE obj, int depth, Out out, bool as_ok) {
                 *out->cur++ = ',';
             }
             *out->cur++ = '"';
-            memcpy(out->cur, name, len);
-            out->cur += len;
+            APPEND_CHARS(out->cur, name, len);
             *out->cur++ = '"';
         }
         *out->cur++ = ']';
     } else {
         fill_indent(out, d3);
         *out->cur++ = '"';
-        memcpy(out->cur, class_name, len);
-        out->cur += len;
+        APPEND_CHARS(out->cur, class_name, len);
         *out->cur++ = '"';
     }
     *out->cur++ = ',';
@@ -730,7 +616,7 @@ static void dump_struct(VALUE obj, int depth, Out out, bool as_ok) {
 
         for (i = 0; i < cnt; i++) {
             v = RSTRUCT_GET(obj, i);
-            if (oj_dump_ignore(out->opts, v)) {
+            if (dump_ignore(out->opts, v)) {
                 v = Qnil;
             }
             assure_size(out, size);
@@ -748,7 +634,7 @@ static void dump_struct(VALUE obj, int depth, Out out, bool as_ok) {
         for (i = 0; i < slen; i++) {
             assure_size(out, size);
             fill_indent(out, d3);
-            if (oj_dump_ignore(out->opts, v)) {
+            if (dump_ignore(out->opts, v)) {
                 v = Qnil;
             }
             oj_dump_obj_val(rb_struct_aref(obj, INT2FIX(i)), d3, out, 0, 0, true);
@@ -757,8 +643,7 @@ static void dump_struct(VALUE obj, int depth, Out out, bool as_ok) {
     }
 #endif
     out->cur--;
-    *out->cur++ = ']';
-    *out->cur++ = '}';
+    APPEND_CHARS(out->cur, "]}", 2);
     *out->cur   = '\0';
 }
 
@@ -798,9 +683,7 @@ static DumpFunc obj_funcs[] = {
 void oj_dump_obj_val(VALUE obj, int depth, Out out) {
     int type = rb_type(obj);
 
-    if (Yes == out->opts->trace) {
-        oj_trace("dump", obj, __FILE__, __LINE__, depth, TraceIn);
-    }
+    TRACE(out->opts->trace, "dump", obj, depth, TraceIn);
     if (MAX_DEPTH < depth) {
         rb_raise(rb_eNoMemError, "Too deeply nested.\n");
     }
@@ -809,14 +692,10 @@ void oj_dump_obj_val(VALUE obj, int depth, Out out) {
 
         if (NULL != f) {
             f(obj, depth, out, false);
-            if (Yes == out->opts->trace) {
-                oj_trace("dump", obj, __FILE__, __LINE__, depth, TraceOut);
-            }
+            TRACE(out->opts->trace, "dump", obj, depth, TraceOut);
             return;
         }
     }
     oj_dump_nil(Qnil, depth, out, false);
-    if (Yes == out->opts->trace) {
-        oj_trace("dump", Qnil, __FILE__, __LINE__, depth, TraceOut);
-    }
+    TRACE(out->opts->trace, "dump", Qnil, depth, TraceOut);
 }
