@@ -11,6 +11,7 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "mem.h"
 #include "encode.h"
 #include "oj.h"
 #include "dump.h"
@@ -74,7 +75,7 @@ static char *read_quoted_value(ParseInfo pi);
 static void  skip_comment(ParseInfo pi);
 
 static VALUE protect_open_proc(VALUE x);
-static VALUE parse_json(VALUE clas, char *json, bool given, bool allocated);
+static VALUE parse_json(VALUE clas, char *json, bool given);
 static void  each_leaf(Doc doc, VALUE self);
 static int   move_step(Doc doc, const char *path, int loc);
 static Leaf  get_doc_leaf(Doc doc, const char *path);
@@ -160,7 +161,7 @@ inline static Leaf leaf_new(Doc doc, int type) {
     Leaf leaf;
 
     if (0 == doc->batches || BATCH_SIZE == doc->batches->next_avail) {
-        Batch b = ALLOC(struct _batch);
+        Batch b = OJ_R_ALLOC(struct _batch);
 
         // Initializes all leaves with a NO_VAL value_type
         memset(b, 0, sizeof(struct _batch));
@@ -648,10 +649,11 @@ static void doc_free(Doc doc) {
         while (0 != (b = doc->batches)) {
             doc->batches = doc->batches->next;
             if (&doc->batch0 != b) {
-                xfree(b);
+                OJ_R_FREE(b);
             }
         }
-        // xfree(f);
+        OJ_R_FREE(doc->json);
+        OJ_R_FREE(doc);
     }
 }
 
@@ -671,27 +673,28 @@ static void free_doc_cb(void *x) {
     Doc doc = (Doc)x;
 
     if (0 != doc) {
-        xfree(doc->json);
         doc_free(doc);
     }
 }
 
 static void mark_leaf(Leaf leaf) {
-    switch (leaf->value_type) {
-    case COL_VAL:
-        if (NULL != leaf->elements) {
-            Leaf first = leaf->elements->next;
-            Leaf e     = first;
+    if (NULL != leaf) {
+        switch (leaf->value_type) {
+        case COL_VAL:
+            if (NULL != leaf->elements) {
+                Leaf first = leaf->elements->next;
+                Leaf e     = first;
 
-            do {
-                mark_leaf(e);
-                e = e->next;
-            } while (e != first);
+                do {
+                    mark_leaf(e);
+                    e = e->next;
+                } while (e != first);
+            }
+            break;
+        case RUBY_VAL: mark(leaf->value); break;
+
+        default: break;
         }
-        break;
-    case RUBY_VAL: mark(leaf->value); break;
-
-    default: break;
     }
 }
 
@@ -747,20 +750,15 @@ static const rb_data_type_t oj_doc_type = {
     0,
 };
 
-static VALUE parse_json(VALUE clas, char *json, bool given, bool allocated) {
+static VALUE parse_json(VALUE clas, char *json, bool given) {
     struct _parseInfo pi;
     volatile VALUE    result = Qnil;
     Doc               doc;
     int               ex = 0;
     volatile VALUE    self;
 
-    // TBD are both needed? is stack allocation ever needed?
+    doc = OJ_R_ALLOC_N(struct _doc, 1);
 
-    if (given) {
-        doc = ALLOCA_N(struct _doc, 1);
-    } else {
-        doc = ALLOC(struct _doc);
-    }
     // skip UTF-8 BOM if present
     if (0xEF == (uint8_t)*json && 0xBB == (uint8_t)json[1] && 0xBF == (uint8_t)json[2]) {
         pi.str = json + 3;
@@ -785,18 +783,20 @@ static VALUE parse_json(VALUE clas, char *json, bool given, bool allocated) {
         }
     }
 #endif
+    doc->json           = json;
     self                = TypedData_Wrap_Struct(clas, &oj_doc_type, doc);
     doc->self           = self;
-    doc->json           = json;
     DATA_PTR(doc->self) = doc;
     result              = rb_protect(protect_open_proc, (VALUE)&pi, &ex);
     if (given || 0 != ex) {
         DATA_PTR(doc->self) = NULL;
+        // TBD is this needed?
+        /*
         doc_free(pi.doc);
-        if (allocated && 0 != ex) {  // will jump so caller will not free
-            xfree(json);
+        if (0 != ex) {  // will jump so caller will not free
+            OJ_R_FREE(json);
         }
-        rb_gc_enable();
+        */
     } else {
         result = doc->self;
     }
@@ -1090,27 +1090,19 @@ static VALUE doc_open(VALUE clas, VALUE str) {
     size_t         len;
     volatile VALUE obj;
     int            given = rb_block_given_p();
-    int            allocate;
 
     Check_Type(str, T_STRING);
     len      = (int)RSTRING_LEN(str) + 1;
-    allocate = (SMALL_JSON < len || !given);
-    if (allocate) {
-        json = ALLOC_N(char, len);
-    } else {
-        json = ALLOCA_N(char, len);
-    }
-    // It should not be necessaary to stop GC but if it is not stopped and a
-    // large string is parsed that string is corrupted or freed during
-    // parsing. I'm not sure what is going on exactly but disabling GC avoids
-    // the issue.
-    rb_gc_disable();
+    json     = OJ_R_ALLOC_N(char, len);
+
     memcpy(json, StringValuePtr(str), len);
-    obj = parse_json(clas, json, given, allocate);
-    rb_gc_enable();
-    if (given && allocate) {
-        xfree(json);
+    obj = parse_json(clas, json, given);
+    // TBD is this needed
+    /*
+    if (given) {
+        OJ_R_FREE(json);
     }
+    */
     return obj;
 }
 
@@ -1140,7 +1132,6 @@ static VALUE doc_open_file(VALUE clas, VALUE filename) {
     size_t         len;
     volatile VALUE obj;
     int            given = rb_block_given_p();
-    int            allocate;
 
     Check_Type(filename, T_STRING);
     path = StringValuePtr(filename);
@@ -1149,12 +1140,8 @@ static VALUE doc_open_file(VALUE clas, VALUE filename) {
     }
     fseek(f, 0, SEEK_END);
     len      = ftell(f);
-    allocate = (SMALL_JSON < len || !given);
-    if (allocate) {
-        json = ALLOC_N(char, len + 1);
-    } else {
-        json = ALLOCA_N(char, len + 1);
-    }
+    json = OJ_R_ALLOC_N(char, len + 1);
+
     fseek(f, 0, SEEK_SET);
     if (len != fread(json, 1, len, f)) {
         fclose(f);
@@ -1165,12 +1152,13 @@ static VALUE doc_open_file(VALUE clas, VALUE filename) {
     }
     fclose(f);
     json[len] = '\0';
-    rb_gc_disable();
-    obj = parse_json(clas, json, given, allocate);
-    rb_gc_enable();
-    if (given && allocate) {
-        xfree(json);
+    obj = parse_json(clas, json, given);
+    // TBD is this needed
+    /*
+    if (given) {
+        OJ_R_FREE(json);
     }
+    */
     return obj;
 }
 
@@ -1654,11 +1642,9 @@ static VALUE doc_close(VALUE self) {
     Doc doc = self_doc(self);
 
     rb_gc_unregister_address(&doc->self);
-    DATA_PTR(doc->self) = 0;
+    DATA_PTR(doc->self) = NULL;
     if (0 != doc) {
-        xfree(doc->json);
         doc_free(doc);
-        xfree(doc);
     }
     return Qnil;
 }
