@@ -8,6 +8,8 @@
 #if HAVE_PTHREAD_MUTEX_INIT
 #include <pthread.h>
 #endif
+
+#include "mem.h"
 #include "cache.h"
 #include "parse.h"
 
@@ -37,13 +39,10 @@ typedef struct _hash {
 struct _hash class_hash;
 struct _hash attr_hash;
 
-static struct _cache *str_cache = NULL;
 static VALUE          str_cache_obj;
 
-static struct _cache *sym_cache = NULL;
 static VALUE          sym_cache_obj;
 
-static struct _cache *attr_cache = NULL;
 static VALUE          attr_cache_obj;
 
 static VALUE form_str(const char *str, size_t len) {
@@ -58,7 +57,7 @@ static VALUE form_attr(const char *str, size_t len) {
     char buf[256];
 
     if (sizeof(buf) - 2 <= len) {
-        char *b = ALLOC_N(char, len + 2);
+        char *b = OJ_R_ALLOC_N(char, len + 2);
         ID    id;
 
         if ('~' == *str) {
@@ -71,7 +70,7 @@ static VALUE form_attr(const char *str, size_t len) {
 	    b[len + 1] = '\0';
 	}
         id = rb_intern3(buf, len + 1, oj_utf8_encoding);
-        xfree(b);
+        OJ_R_FREE(b);
         return id;
     }
     if ('~' == *str) {
@@ -93,15 +92,15 @@ void oj_hash_init(void) {
     rb_gc_register_address(&cache_class);
     rb_undef_alloc_func(cache_class);
 
-    str_cache     = cache_create(0, form_str, true, true);
+    struct _cache *str_cache     = cache_create(0, form_str, true, true);
     str_cache_obj = Data_Wrap_Struct(cache_class, cache_mark, cache_free, str_cache);
     rb_gc_register_address(&str_cache_obj);
 
-    sym_cache     = cache_create(0, form_sym, true, true);
+    struct _cache *sym_cache     = cache_create(0, form_sym, true, true);
     sym_cache_obj = Data_Wrap_Struct(cache_class, cache_mark, cache_free, sym_cache);
     rb_gc_register_address(&sym_cache_obj);
 
-    attr_cache     = cache_create(0, form_attr, false, true);
+    struct _cache *attr_cache = cache_create(0, form_attr, false, true);
     attr_cache_obj = Data_Wrap_Struct(cache_class, cache_mark, cache_free, attr_cache);
     rb_gc_register_address(&attr_cache_obj);
 
@@ -122,18 +121,18 @@ oj_str_intern(const char *key, size_t len) {
 #if HAVE_RB_ENC_INTERNED_STR && 0
     return rb_enc_interned_str(key, len, rb_utf8_encoding());
 #else
-    return cache_intern(str_cache, key, len);
+    return cache_intern(DATA_PTR(str_cache_obj), key, len);
 #endif
 }
 
 VALUE
 oj_sym_intern(const char *key, size_t len) {
-    return cache_intern(sym_cache, key, len);
+  return cache_intern(DATA_PTR(sym_cache_obj), key, len);
 }
 
 ID
 oj_attr_intern(const char *key, size_t len) {
-    return cache_intern(attr_cache, key, len);
+  return cache_intern(DATA_PTR(attr_cache_obj), key, len);
 }
 
 static uint64_t hash_calc(const uint8_t *key, size_t len) {
@@ -249,7 +248,7 @@ VALUE oj_class_intern(const char *key, size_t len, bool safe, ParseInfo pi, int
                 }
                 bucket = b;
             }
-            b            = ALLOC(struct _keyVal);
+            b            = OJ_R_ALLOC(struct _keyVal);
             b->next      = NULL;
             bucket->next = b;
             bucket       = b;
@@ -270,7 +269,7 @@ VALUE oj_class_intern(const char *key, size_t len, bool safe, ParseInfo pi, int
                 }
                 bucket = b;
             }
-            b            = ALLOC(struct _keyVal);
+            b            = OJ_R_ALLOC(struct _keyVal);
             b->next      = NULL;
             bucket->next = b;
             bucket       = b;
@@ -284,7 +283,7 @@ VALUE oj_class_intern(const char *key, size_t len, bool safe, ParseInfo pi, int
 }
 
 char *oj_strndup(const char *s, size_t len) {
-    char *d = ALLOC_N(char, len + 1);
+    char *d = OJ_R_ALLOC_N(char, len + 1);
 
     memcpy(d, s, len);
     d[len] = '\0';
