@@ -11,6 +11,7 @@
 #include <sys/types.h>
 #include <unistd.h>
 
+#include "mem.h"
 #include "dump.h"
 #include "encode.h"
 #include "intern.h"
@@ -32,6 +33,7 @@ ID oj_array_append_id;
 ID oj_array_end_id;
 ID oj_array_start_id;
 ID oj_as_json_id;
+ID oj_at_id;
 ID oj_begin_id;
 ID oj_bigdecimal_id;
 ID oj_end_id;
@@ -89,7 +91,9 @@ VALUE oj_array_class_sym;
 VALUE oj_create_additions_sym;
 VALUE oj_decimal_class_sym;
 VALUE oj_hash_class_sym;
+VALUE oj_in_sym;
 VALUE oj_indent_sym;
+VALUE oj_nanosecond_sym;
 VALUE oj_object_class_sym;
 VALUE oj_quirks_mode_sym;
 VALUE oj_safe_sym;
@@ -240,7 +244,7 @@ struct _options oj_default_options = {
  *references
  * - *:auto_define* [_Boolean_|_nil_] automatically define classes if they do not exist
  * - *:symbol_keys* [_Boolean_|_nil_] use symbols instead of strings for hash keys
- * - *:escape_mode* [_:newline_|_:json_|_:xss_safe_|_:ascii_|_unicode_xss_|_nil_] determines the
+ * - *:escape_mode* [_:newline_|_:json_|_:slash_|_:xss_safe_|_:ascii_|_:unicode_xss_|_nil_] determines the
  *characters to escape
  * - *:class_cache* [_Boolean_|_nil_] cache classes for faster parsing (if dynamically modifying
  *classes or reloading classes then don't use this)
@@ -248,7 +252,7 @@ struct _options oj_default_options = {
  *to use for JSON
  * - *:time_format* [_:unix_|_:unix_zone_|_:xmlschema_|_:ruby_] time format when dumping
  * - *:bigdecimal_as_decimal* [_Boolean_|_nil_] dump BigDecimal as a decimal number or as a String
- * - *:bigdecimal_load* [_:bigdecimal_|_:float_|_:auto_|_:fast_] load decimals as BigDecimal instead
+ * - *:bigdecimal_load* [_:bigdecimal_|_:float_|_:auto_|_:fast_|_:ruby_] load decimals as BigDecimal instead
  *of as a Float. :auto pick the most precise for the number of digits. :float should be the same as
  *ruby. :fast may require rounding but is must faster.
  * - *:compat_bigdecimal* [_true_|_false_] load decimals as BigDecimal instead of as a Float when in
@@ -779,7 +783,7 @@ static int parse_options_cb(VALUE k, VALUE v, VALUE opts) {
     } else if (create_id_sym == k) {
         if (Qnil == v) {
             if (oj_json_class != oj_default_options.create_id && NULL != copts->create_id) {
-                xfree((char *)oj_default_options.create_id);
+                OJ_R_FREE((char *)oj_default_options.create_id);
             }
             copts->create_id     = NULL;
             copts->create_id_len = 0;
@@ -788,7 +792,7 @@ static int parse_options_cb(VALUE k, VALUE v, VALUE opts) {
 
             len = RSTRING_LEN(v);
             if (len != copts->create_id_len || 0 != strcmp(copts->create_id, str)) {
-                copts->create_id = ALLOC_N(char, len + 1);
+                copts->create_id = OJ_R_ALLOC_N(char, len + 1);
                 strcpy((char *)copts->create_id, str);
                 copts->create_id_len = len;
             }
@@ -908,7 +912,7 @@ static int parse_options_cb(VALUE k, VALUE v, VALUE opts) {
             copts->array_class = v;
         }
     } else if (ignore_sym == k) {
-        xfree(copts->ignore);
+        OJ_R_FREE(copts->ignore);
         copts->ignore = NULL;
         if (Qnil != v) {
             int cnt;
@@ -918,7 +922,7 @@ static int parse_options_cb(VALUE k, VALUE v, VALUE opts) {
             if (0 < cnt) {
                 int i;
 
-                copts->ignore = ALLOC_N(VALUE, cnt + 1);
+                copts->ignore = OJ_R_ALLOC_N(VALUE, cnt + 1);
                 for (i = 0; i < cnt; i++) {
                     copts->ignore[i] = RARRAY_AREF(v, i);
                 }
@@ -929,7 +933,7 @@ static int parse_options_cb(VALUE k, VALUE v, VALUE opts) {
         if (Qnil == v) {
             return ST_CONTINUE;
         }
-        if (TYPE(v) == T_STRUCT && rb_obj_class(v) == rb_cRange) {
+        if (rb_obj_class(v) == rb_cRange) {
             VALUE min = rb_funcall(v, oj_begin_id, 0);
             VALUE max = rb_funcall(v, oj_end_id, 0);
 
@@ -1156,7 +1160,7 @@ static VALUE load_file(int argc, VALUE *argv, VALUE self) {
         WCHAR *wide_path;
         wide_path = rb_w32_mbstr_to_wstr(CP_UTF8, path, -1, NULL);
         fd = rb_w32_wopen(wide_path, O_RDONLY);
-        free(wide_path);
+        OJ_FREE(wide_path);
     }
 #else
     fd = open(path, O_RDONLY);
@@ -1721,6 +1725,10 @@ debug_odd(VALUE self, VALUE label) {
     return Qnil;
 }
 
+static VALUE mem_report(VALUE self) {
+    oj_mem_report();
+    return Qnil;
+}
 
 /* Document-module: Oj
  *
@@ -1809,11 +1817,14 @@ void Init_oj(void) {
 
     rb_define_module_function(Oj, "optimize_rails", oj_optimize_rails, 0);
 
+    rb_define_module_function(Oj, "mem_report", mem_report, 0);
+
     oj_add_value_id          = rb_intern("add_value");
     oj_array_append_id       = rb_intern("array_append");
     oj_array_end_id          = rb_intern("array_end");
     oj_array_start_id        = rb_intern("array_start");
     oj_as_json_id            = rb_intern("as_json");
+    oj_at_id                 = rb_intern("at");
     oj_begin_id              = rb_intern("begin");
     oj_bigdecimal_id         = rb_intern("BigDecimal");
     oj_end_id                = rb_intern("end");
@@ -1961,10 +1972,14 @@ void Init_oj(void) {
     rb_gc_register_address(&oj_decimal_class_sym);
     oj_hash_class_sym = ID2SYM(rb_intern("hash_class"));
     rb_gc_register_address(&oj_hash_class_sym);
+    oj_in_sym = ID2SYM(rb_intern("in"));
+    rb_gc_register_address(&oj_in_sym);
     oj_indent_sym = ID2SYM(rb_intern("indent"));
     rb_gc_register_address(&oj_indent_sym);
     oj_max_nesting_sym = ID2SYM(rb_intern("max_nesting"));
     rb_gc_register_address(&oj_max_nesting_sym);
+    oj_nanosecond_sym = ID2SYM(rb_intern("nanosecond"));
+    rb_gc_register_address(&oj_nanosecond_sym);
     oj_object_class_sym = ID2SYM(rb_intern("object_class"));
     rb_gc_register_address(&oj_object_class_sym);
     oj_object_nl_sym = ID2SYM(rb_intern("object_nl"));
