@@ -317,7 +317,9 @@ extern VALUE oj_ascii_only_sym;
 extern VALUE oj_create_additions_sym;
 extern VALUE oj_decimal_class_sym;
 extern VALUE oj_hash_class_sym;
+extern VALUE oj_in_sym;
 extern VALUE oj_indent_sym;
+extern VALUE oj_nanosecond_sym;
 extern VALUE oj_max_nesting_sym;
 extern VALUE oj_object_class_sym;
 extern VALUE oj_object_nl_sym;
@@ -334,6 +336,7 @@ extern ID oj_array_append_id;
 extern ID oj_array_end_id;
 extern ID oj_array_start_id;
 extern ID oj_as_json_id;
+extern ID oj_at_id;
 extern ID oj_begin_id;
 extern ID oj_bigdecimal_id;
 extern ID oj_end_id;
@@ -376,6 +379,12 @@ extern bool oj_use_hash_alt;
 extern bool oj_use_array_alt;
 extern bool string_writer_optimized;
 
+static inline VALUE oj_safe_string_convert(VALUE obj) {
+    VALUE rstr = rb_funcall(obj, oj_to_s_id, 0);
+    StringValue(rstr);
+    return rstr;
+}
+
 #define APPEND_CHARS(buffer, chars, size) \
     { \
         memcpy(buffer, chars, size); \
