@@ -10,12 +10,17 @@
 #include <string.h>
 #include <unistd.h>
 
+#include "mem.h"
 #include "buf.h"
 #include "encode.h"
 #include "oj.h"
 #include "rxclass.h"
 #include "val_stack.h"
 
+#ifdef OJ_USE_SSE4_2
+#include <nmmintrin.h>
+#endif
+
 // Workaround in case INFINITY is not defined in math.h or if the OS is CentOS
 #define OJ_INFINITY (1.0 / 0.0)
 
@@ -83,7 +88,7 @@ static void add_value(ParseInfo pi, VALUE rval) {
             pi->hash_set_value(pi, parent, rval);
             if (0 != parent->key && 0 < parent->klen &&
                 (parent->key < pi->json || pi->cur < parent->key)) {
-                xfree((char *)parent->key);
+                OJ_R_FREE((char *)parent->key);
                 parent->key = 0;
             }
             parent->next = NEXT_HASH_COMMA;
@@ -192,32 +197,7 @@ static inline const char *scan_string_noSIMD(const char *str, const char *end) {
     return str;
 }
 
-// Taken from Tensorflow:
-// https://github.com/tensorflow/tensorflow/blob/5dcfc51118817f27fad5246812d83e5dccdc5f72/tensorflow/core/lib/hash/crc32c_accelerate.cc#L21-L38
-#ifdef __SSE4_2__
-#if defined(__x86_64__) && defined(__GNUC__) && \
-    (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 8))
-#define USE_SSE_DETECT 1
-#elif defined(__x86_64__) && defined(__clang__)
-#if __has_builtin(__builtin_cpu_supports)
-#define USE_SSE_DETECT 1
-#endif
-#endif
-#endif /* __SSE4_2__ */
-
-// This version of Apple clang has a bug:
-// https://llvm.org/bugs/show_bug.cgi?id=25510
-#if defined(__APPLE__) && (__clang_major__ <= 8)
-#undef USE_SSE_DETECT
-#endif
-
-#if defined(TRUFFLERUBY)
-#undef USE_SSE_DETECT
-#endif
-
-#ifdef USE_SSE_DETECT
-#include <nmmintrin.h>
-
+#ifdef OJ_USE_SSE4_2
 static inline const char *scan_string_SIMD(const char *str, const char *end) {
     static const char chars[16] = "\x00\\\"";
     const __m128i terminate = _mm_loadu_si128((const __m128i *)&chars[0]);
@@ -236,23 +216,12 @@ static inline const char *scan_string_SIMD(const char *str, const char *end) {
 }
 #endif
 
-static bool cpu_supports_sse42(void) {
-#if USE_SSE_DETECT
-    __builtin_cpu_init();
-    return (__builtin_cpu_supports("sse4.2"));
-#else
-    return false;
-#endif
-}
-
 static const char *(*scan_func) (const char *str, const char *end) = scan_string_noSIMD;
 
 void oj_scanner_init(void) {
-    if (cpu_supports_sse42()) {
-#if USE_SSE_DETECT
-        scan_func = scan_string_SIMD;
+#ifdef OJ_USE_SSE4_2
+    scan_func = scan_string_SIMD;
 #endif
-    }
 }
 
 // entered at /
@@ -268,7 +237,8 @@ static void read_escaped_str(ParseInfo pi, const char *start) {
 
     for (s = pi->cur; '"' != *s;) {
         const char *scanned = scan_func(s, pi->end);
-        if (scanned >= pi->end) {
+        if (scanned >= pi->end || '\0' == *scanned) {
+            //if (scanned >= pi->end) {
             oj_set_error_at(pi,
                             oj_parse_error_class,
                             __FILE__,
@@ -277,7 +247,6 @@ static void read_escaped_str(ParseInfo pi, const char *start) {
             buf_cleanup(&buf);
             return;
         }
-
         buf_append_string(&buf, s, (size_t)(scanned - s));
         s = scanned;
 
@@ -366,7 +335,7 @@ static void read_escaped_str(ParseInfo pi, const char *start) {
         case NEXT_HASH_KEY:
             if (Qundef == (parent->key_val = pi->hash_key(pi, buf.head, buf_len(&buf)))) {
                 parent->klen = buf_len(&buf);
-                parent->key  = malloc(parent->klen + 1);
+                parent->key  = OJ_MALLOC(parent->klen + 1);
                 memcpy((char *)parent->key, buf.head, parent->klen);
                 *(char *)(parent->key + parent->klen) = '\0';
             } else {
@@ -380,7 +349,7 @@ static void read_escaped_str(ParseInfo pi, const char *start) {
             pi->hash_set_cstr(pi, parent, buf.head, buf_len(&buf), start);
             if (0 != parent->key && 0 < parent->klen &&
                 (parent->key < pi->json || pi->cur < parent->key)) {
-                xfree((char *)parent->key);
+                OJ_R_FREE((char *)parent->key);
                 parent->key = 0;
             }
             parent->next = NEXT_HASH_COMMA;
@@ -450,7 +419,7 @@ static void read_str(ParseInfo pi) {
             pi->hash_set_cstr(pi, parent, str, pi->cur - str, str);
             if (0 != parent->key && 0 < parent->klen &&
                 (parent->key < pi->json || pi->cur < parent->key)) {
-                xfree((char *)parent->key);
+                OJ_R_FREE((char *)parent->key);
                 parent->key = 0;
             }
             parent->next = NEXT_HASH_COMMA;
@@ -649,7 +618,7 @@ static void read_num(ParseInfo pi) {
             pi->hash_set_num(pi, parent, &ni);
             if (0 != parent->key && 0 < parent->klen &&
                 (parent->key < pi->json || pi->cur < parent->key)) {
-                xfree((char *)parent->key);
+                OJ_R_FREE((char *)parent->key);
                 parent->key = 0;
             }
             parent->next = NEXT_HASH_COMMA;
@@ -667,7 +636,7 @@ static void read_num(ParseInfo pi) {
 }
 
 static void array_start(ParseInfo pi) {
-    volatile VALUE v = pi->start_array(pi);
+    VALUE v = pi->start_array(pi);
 
     stack_push(&pi->stack, v, NEXT_ARRAY_NEW);
 }
@@ -691,13 +660,13 @@ static void array_end(ParseInfo pi) {
 }
 
 static void hash_start(ParseInfo pi) {
-    volatile VALUE v = pi->start_hash(pi);
+    VALUE v = pi->start_hash(pi);
 
     stack_push(&pi->stack, v, NEXT_HASH_NEW);
 }
 
 static void hash_end(ParseInfo pi) {
-    volatile Val hash = stack_peek(&pi->stack);
+    Val hash = stack_peek(&pi->stack);
 
     // leave hash on stack until just before
     if (0 == hash) {
@@ -884,7 +853,7 @@ static long double exp_plus[] = {
 
 VALUE
 oj_num_as_value(NumInfo ni) {
-    volatile VALUE rnum = Qnil;
+    VALUE rnum = Qnil;
 
     if (ni->infinity) {
         if (ni->neg) {
@@ -903,12 +872,12 @@ oj_num_as_value(NumInfo ni) {
                 buf[ni->len] = '\0';
                 rnum         = rb_cstr_to_inum(buf, 10, 0);
             } else {
-                char *buf = ALLOC_N(char, ni->len + 1);
+                char *buf = OJ_R_ALLOC_N(char, ni->len + 1);
 
                 memcpy(buf, ni->str, ni->len);
                 buf[ni->len] = '\0';
                 rnum         = rb_cstr_to_inum(buf, 10, 0);
-                xfree(buf);
+                OJ_R_FREE(buf);
             }
         } else {
             if (ni->neg) {
@@ -919,7 +888,7 @@ oj_num_as_value(NumInfo ni) {
         }
     } else {  // decimal
         if (ni->big) {
-            volatile VALUE bd = rb_str_new(ni->str, ni->len);
+            VALUE bd = rb_str_new(ni->str, ni->len);
 
             rnum = rb_rescue2(parse_big_decimal, bd, rescue_big_decimal, bd, rb_eException, 0);
             if (ni->no_big) {
@@ -947,7 +916,7 @@ oj_num_as_value(NumInfo ni) {
             }
             rnum = rb_float_new((double)ld);
         } else if (RubyDec == ni->bigdec_load) {
-            volatile VALUE sv = rb_str_new(ni->str, ni->len);
+            VALUE sv = rb_str_new(ni->str, ni->len);
 
             rnum = rb_funcall(sv, rb_intern("to_f"), 0);
         } else {
@@ -1042,7 +1011,7 @@ static VALUE protect_parse(VALUE pip) {
 
 extern int oj_utf8_index;
 
-static void oj_pi_set_input_str(ParseInfo pi, volatile VALUE *inputp) {
+static void oj_pi_set_input_str(ParseInfo pi, VALUE *inputp) {
     int idx = RB_ENCODING_GET(*inputp);
 
     if (oj_utf8_encoding_index != idx) {
@@ -1055,12 +1024,12 @@ static void oj_pi_set_input_str(ParseInfo pi, volatile VALUE *inputp) {
 
 VALUE
 oj_pi_parse(int argc, VALUE *argv, ParseInfo pi, char *json, size_t len, int yieldOk) {
-    char *         buf = 0;
-    volatile VALUE input;
-    volatile VALUE wrapped_stack;
-    volatile VALUE result    = Qnil;
-    int            line      = 0;
-    int            free_json = 0;
+    char * buf = 0;
+    VALUE  input;
+    VALUE  wrapped_stack;
+    VALUE  result    = Qnil;
+    int    line      = 0;
+    int    free_json = 0;
 
     if (argc < 1) {
         rb_raise(rb_eArgError, "Wrong number of arguments to parse.");
@@ -1096,8 +1065,8 @@ oj_pi_parse(int argc, VALUE *argv, ParseInfo pi, char *json, size_t len, int yie
             rb_raise(rb_eTypeError, "Nil is not a valid JSON source.");
         }
     } else {
-        VALUE          clas = rb_obj_class(input);
-        volatile VALUE s;
+        VALUE clas = rb_obj_class(input);
+        VALUE s;
 
         if (oj_stringio_class == clas) {
             s = rb_funcall2(input, oj_string_id, 0, 0);
@@ -1109,12 +1078,12 @@ oj_pi_parse(int argc, VALUE *argv, ParseInfo pi, char *json, size_t len, int yie
             size_t  len = lseek(fd, 0, SEEK_END);
 
             lseek(fd, 0, SEEK_SET);
-            buf      = ALLOC_N(char, len + 1);
+            buf      = OJ_R_ALLOC_N(char, len + 1);
             pi->json = buf;
             pi->end  = buf + len;
             if (0 >= (cnt = read(fd, (char *)pi->json, len)) || cnt != (ssize_t)len) {
                 if (0 != buf) {
-                    xfree(buf);
+                    OJ_R_FREE(buf);
                 }
                 rb_raise(rb_eIOError, "failed to read from IO Object.");
             }
@@ -1196,9 +1165,9 @@ CLEANUP:
         oj_circ_array_free(pi->circ_array);
     }
     if (0 != buf) {
-        xfree(buf);
+        OJ_R_FREE(buf);
     } else if (free_json) {
-        xfree(json);
+        OJ_R_FREE(json);
     }
     stack_cleanup(&pi->stack);
     if (pi->str_rx.head != oj_default_options.str_rx.head) {
