@@ -1145,8 +1145,10 @@ static void parser_free(void *ptr) {
     p = (ojParser)ptr;
     buf_cleanup(&p->key);
     buf_cleanup(&p->buf);
-    p->free(p);
-    xfree(ptr);
+    if (NULL != p->free) {
+        p->free(p);
+    }
+    OJ_R_FREE(ptr);
 }
 
 static void parser_mark(void *ptr) {
@@ -1156,7 +1158,9 @@ static void parser_mark(void *ptr) {
         if (0 != p->reader) {
             rb_gc_mark(p->reader);
         }
-        p->mark(p);
+        if (NULL != p->mark) {
+            p->mark(p);
+        }
     }
 }
 
@@ -1201,7 +1205,7 @@ static int opt_cb(VALUE rkey, VALUE value, VALUE ptr) {
  * Oj::Parser.new(:usual, cache_keys: true).
  */
 static VALUE parser_new(int argc, VALUE *argv, VALUE self) {
-    ojParser p = ALLOC(struct _ojParser);
+    ojParser p = OJ_R_ALLOC(struct _ojParser);
 
 #if HAVE_RB_EXT_RACTOR_SAFE
     // This doesn't seem to do anything.
@@ -1254,6 +1258,32 @@ static VALUE parser_new(int argc, VALUE *argv, VALUE self) {
     return Data_Wrap_Struct(parser_class, parser_mark, parser_free, p);
 }
 
+// Create a new parser without setting the delegate. The parser is
+// wrapped. The parser is (ojParser)DATA_PTR(value) where value is the return
+// from this function. A delegate must be added before the parser can be
+// used. Optionally oj_parser_set_options can be called if the options are not
+// set directly.
+VALUE oj_parser_new(void) {
+    ojParser p = OJ_R_ALLOC(struct _ojParser);
+
+#if HAVE_RB_EXT_RACTOR_SAFE
+    // This doesn't seem to do anything.
+    rb_ext_ractor_safe(true);
+#endif
+    memset(p, 0, sizeof(struct _ojParser));
+    buf_init(&p->key);
+    buf_init(&p->buf);
+    p->map = value_map;
+
+    return Data_Wrap_Struct(parser_class, parser_mark, parser_free, p);
+}
+
+// Set set the options from a hash (ropts).
+void oj_parser_set_option(ojParser p, VALUE ropts) {
+    Check_Type(ropts, T_HASH);
+    rb_hash_foreach(ropts, opt_cb, (VALUE)p);
+}
+
 /* Document-method: method_missing(value)
  * call-seq: method_missing(value)
  *
@@ -1469,7 +1499,7 @@ static VALUE usual_parser = Qundef;
  */
 static VALUE parser_usual(VALUE self) {
     if (Qundef == usual_parser) {
-        ojParser p = ALLOC(struct _ojParser);
+        ojParser p = OJ_R_ALLOC(struct _ojParser);
 
         memset(p, 0, sizeof(struct _ojParser));
         buf_init(&p->key);
@@ -1492,7 +1522,7 @@ static VALUE saj_parser = Qundef;
  */
 static VALUE parser_saj(VALUE self) {
     if (Qundef == saj_parser) {
-        ojParser p = ALLOC(struct _ojParser);
+        ojParser p = OJ_R_ALLOC(struct _ojParser);
 
         memset(p, 0, sizeof(struct _ojParser));
         buf_init(&p->key);
@@ -1514,7 +1544,7 @@ static VALUE validate_parser = Qundef;
  */
 static VALUE parser_validate(VALUE self) {
     if (Qundef == validate_parser) {
-        ojParser p = ALLOC(struct _ojParser);
+        ojParser p = OJ_R_ALLOC(struct _ojParser);
 
         memset(p, 0, sizeof(struct _ojParser));
         buf_init(&p->key);
