@@ -3,6 +3,7 @@
 
 #include "rails.h"
 
+#include "mem.h"
 #include "code.h"
 #include "encode.h"
 #include "trace.h"
@@ -76,7 +77,7 @@ static ROptTable copy_opts(ROptTable src, ROptTable dest) {
     if (NULL == src->table) {
         dest->table = NULL;
     } else {
-        dest->table = ALLOC_N(struct _rOpt, dest->alen);
+        dest->table = OJ_R_ALLOC_N(struct _rOpt, dest->alen);
         memcpy(dest->table, src->table, sizeof(struct _rOpt) * dest->alen);
     }
     return NULL;
@@ -198,7 +199,7 @@ static void dump_enumerable(VALUE obj, int depth, Out out, bool as_ok) {
 }
 
 static void dump_bigdecimal(VALUE obj, int depth, Out out, bool as_ok) {
-    volatile VALUE rstr = rb_funcall(obj, oj_to_s_id, 0);
+    volatile VALUE rstr = oj_safe_string_convert(obj);
     const char *   str  = RSTRING_PTR(rstr);
 
     if ('I' == *str || 'N' == *str || ('-' == *str && 'I' == str[1])) {
@@ -345,7 +346,7 @@ static void dump_timewithzone(VALUE obj, int depth, Out out, bool as_ok) {
 }
 
 static void dump_to_s(VALUE obj, int depth, Out out, bool as_ok) {
-    volatile VALUE rstr = rb_funcall(obj, oj_to_s_id, 0);
+    volatile VALUE rstr = oj_safe_string_convert(obj);
 
     oj_dump_cstr(RSTRING_PTR(rstr), (int)RSTRING_LEN(rstr), 0, 0, out);
 }
@@ -373,11 +374,11 @@ static StrLen columns_array(VALUE rcols, int *ccnt) {
     int            cnt = (int)RARRAY_LEN(rcols);
 
     *ccnt = cnt;
-    cols  = ALLOC_N(struct _strLen, cnt);
+    cols  = OJ_R_ALLOC_N(struct _strLen, cnt);
     for (i = 0, cp = cols; i < cnt; i++, cp++) {
         v = RARRAY_AREF(rcols, i);
         if (T_STRING != rb_type(v)) {
-            v = rb_funcall(v, oj_to_s_id, 0);
+            v = oj_safe_string_convert(v);
         }
         cp->str = StringValuePtr(v);
         cp->len = (int)RSTRING_LEN(v);
@@ -481,7 +482,7 @@ static void dump_activerecord_result(VALUE obj, int depth, Out out, bool as_ok)
             *out->cur++ = ',';
         }
     }
-    xfree(cols);
+    OJ_R_FREE(cols);
     size = depth * out->indent + 1;
     assure_size(out, size);
     if (out->opts->dump_opts.use) {
@@ -517,9 +518,7 @@ static void dump_as_string(VALUE obj, int depth, Out out, bool as_ok) {
 static void dump_as_json(VALUE obj, int depth, Out out, bool as_ok) {
     volatile VALUE ja;
 
-    if (RB_UNLIKELY(Yes == out->opts->trace)) {
-        oj_trace("as_json", obj, __FILE__, __LINE__, depth + 1, TraceRubyIn);
-    }
+    TRACE(out->opts->trace, "as_json", obj, depth + 1, TraceRubyIn);
     // Some classes elect to not take an options argument so check the arity
     // of as_json.
     if (0 == rb_obj_method_arity(obj, oj_as_json_id)) {
@@ -527,9 +526,7 @@ static void dump_as_json(VALUE obj, int depth, Out out, bool as_ok) {
     } else {
         ja = rb_funcall2(obj, oj_as_json_id, out->argc, out->argv);
     }
-    if (RB_UNLIKELY(Yes == out->opts->trace)) {
-        oj_trace("as_json", obj, __FILE__, __LINE__, depth + 1, TraceRubyOut);
-    }
+    TRACE(out->opts->trace, "as_json", obj, depth + 1, TraceRubyOut);
 
     out->argc = 0;
     if (ja == obj || !as_ok) {
@@ -587,11 +584,11 @@ static ROpt create_opt(ROptTable rot, VALUE clas) {
     rot->len++;
     if (NULL == rot->table) {
         rot->alen  = 256;
-        rot->table = ALLOC_N(struct _rOpt, rot->alen);
+        rot->table = OJ_R_ALLOC_N(struct _rOpt, rot->alen);
         memset(rot->table, 0, sizeof(struct _rOpt) * rot->alen);
     } else if (rot->alen <= rot->len) {
         rot->alen *= 2;
-        REALLOC_N(rot->table, struct _rOpt, rot->alen);
+        OJ_R_REALLOC_N(rot->table, struct _rOpt, rot->alen);
         memset(rot->table + olen, 0, sizeof(struct _rOpt) * olen);
     }
     if (0 == olen) {
@@ -644,9 +641,9 @@ static void encoder_free(void *ptr) {
         Encoder e = (Encoder)ptr;
 
         if (NULL != e->ropts.table) {
-            xfree(e->ropts.table);
+            OJ_R_FREE(e->ropts.table);
         }
-        xfree(ptr);
+        OJ_R_FREE(ptr);
     }
 }
 
@@ -667,7 +664,7 @@ static void encoder_mark(void *ptr) {
  * - *options* [_Hash_] formatting options
  */
 static VALUE encoder_new(int argc, VALUE *argv, VALUE self) {
-    Encoder e = ALLOC(struct _encoder);
+    Encoder e = OJ_R_ALLOC(struct _encoder);
 
     e->opts = oj_default_options;
     e->arg  = Qnil;
@@ -1208,7 +1205,7 @@ static void dump_float(VALUE obj, int depth, Out out, bool as_ok) {
         } else if (oj_rails_float_opt) {
             cnt = oj_dump_float_printf(buf, sizeof(buf), obj, d, "%0.16g");
         } else {
-            volatile VALUE rstr = rb_funcall(obj, oj_to_s_id, 0);
+            volatile VALUE rstr = oj_safe_string_convert(obj);
 
             strcpy(buf, RSTRING_PTR(rstr));
             cnt = (int)RSTRING_LEN(rstr);
@@ -1301,7 +1298,7 @@ static int hash_cb(VALUE key, VALUE value, VALUE ov) {
         return ST_CONTINUE;
     }
     if (rtype != T_STRING && rtype != T_SYMBOL) {
-        key   = rb_funcall(key, oj_to_s_id, 0);
+        key   = oj_safe_string_convert(key);
         rtype = rb_type(key);
     }
     if (!out->opts->dump_opts.use) {
@@ -1464,9 +1461,7 @@ static DumpFunc rails_funcs[] = {
 static void dump_rails_val(VALUE obj, int depth, Out out, bool as_ok) {
     int type = rb_type(obj);
 
-    if (RB_UNLIKELY(Yes == out->opts->trace)) {
-        oj_trace("dump", obj, __FILE__, __LINE__, depth, TraceIn);
-    }
+    TRACE(out->opts->trace, "dump", obj, depth, TraceIn);
     if (MAX_DEPTH < depth) {
         rb_raise(rb_eNoMemError, "Too deeply nested.\n");
     }
@@ -1475,16 +1470,12 @@ static void dump_rails_val(VALUE obj, int depth, Out out, bool as_ok) {
 
         if (NULL != f) {
             f(obj, depth, out, as_ok);
-            if (RB_UNLIKELY(Yes == out->opts->trace)) {
-                oj_trace("dump", obj, __FILE__, __LINE__, depth, TraceOut);
-            }
+            TRACE(out->opts->trace, "dump", obj, depth, TraceOut);
             return;
         }
     }
     oj_dump_nil(Qnil, depth, out, false);
-    if (RB_UNLIKELY(Yes == out->opts->trace)) {
-        oj_trace("dump", Qnil, __FILE__, __LINE__, depth, TraceOut);
-    }
+    TRACE(out->opts->trace, "dump", Qnil, depth, TraceOut);
 }
 
 void oj_dump_rails_val(VALUE obj, int depth, Out out) {
