@@ -14,6 +14,7 @@
 // Workaround in case INFINITY is not defined in math.h or if the OS is CentOS
 #define OJ_INFINITY (1.0 / 0.0)
 
+#include "mem.h"
 #include "encode.h"
 #include "oj.h"
 
@@ -210,10 +211,7 @@ static void read_hash(ParseInfo pi, const char *key) {
                 pi->s++;
             } else {
                 if (pi->has_error) {
-                    call_error("invalid format, expected , or } while in an object",
-                               pi,
-                               __FILE__,
-                               __LINE__);
+                    call_error("invalid format, expected , or } while in an object", pi, __FILE__, __LINE__);
                 }
                 raise_error("invalid format, expected , or } while in an object", pi->str, pi->s);
             }
@@ -243,10 +241,7 @@ static void read_array(ParseInfo pi, const char *key) {
                 break;
             } else {
                 if (pi->has_error) {
-                    call_error("invalid format, expected , or ] while in an array",
-                               pi,
-                               __FILE__,
-                               __LINE__);
+                    call_error("invalid format, expected , or ] while in an array", pi, __FILE__, __LINE__);
                 }
                 raise_error("invalid format, expected , or ] while in an array", pi->str, pi->s);
             }
@@ -276,7 +271,7 @@ static void read_str(ParseInfo pi, const char *key) {
 #endif
 
 static void read_num(ParseInfo pi, const char *key) {
-    char *  start = pi->s;
+    char   *start = pi->s;
     int64_t n     = 0;
     long    a     = 0;
     long    div   = 1;
@@ -351,9 +346,7 @@ static void read_num(ParseInfo pi, const char *key) {
 
             *pi->s = '\0';
             if (pi->has_add_value) {
-                call_add_value(pi->handler,
-                               rb_funcall(rb_cObject, oj_bigdecimal_id, 1, rb_str_new2(start)),
-                               key);
+                call_add_value(pi->handler, rb_funcall(rb_cObject, oj_bigdecimal_id, 1, rb_str_new2(start)), key);
             }
             *pi->s = c;
         } else {
@@ -371,9 +364,7 @@ static void read_num(ParseInfo pi, const char *key) {
 
             *pi->s = '\0';
             if (pi->has_add_value) {
-                call_add_value(pi->handler,
-                               rb_funcall(rb_cObject, oj_bigdecimal_id, 1, rb_str_new2(start)),
-                               key);
+                call_add_value(pi->handler, rb_funcall(rb_cObject, oj_bigdecimal_id, 1, rb_str_new2(start)), key);
             }
             *pi->s = c;
         } else {
@@ -505,9 +496,9 @@ static char *unicode_to_chars(ParseInfo pi, char *t, uint32_t code) {
  * reached again. Do not read the character after the terminating quote.
  */
 static char *read_quoted_value(ParseInfo pi) {
-    char *   value = 0;
-    char *   h     = pi->s; /* head */
-    char *   t     = h;     /* tail */
+    char    *value = 0;
+    char    *h     = pi->s; /* head */
+    char    *t     = h;     /* tail */
     uint32_t code;
 
     h++; /* skip quote character */
@@ -588,15 +579,13 @@ static void saj_parse(VALUE handler, char *json) {
     pi.str = json;
     pi.s   = json;
 #if IS_WINDOWS
-    pi.stack_min = (void *)((char *)&obj -
-                            (512 * 1024)); /* assume a 1M stack and give half to ruby */
+    pi.stack_min = (void *)((char *)&obj - (512L * 1024L)); /* assume a 1M stack and give half to ruby */
 #else
     {
         struct rlimit lim;
 
         if (0 == getrlimit(RLIMIT_STACK, &lim) && RLIM_INFINITY != lim.rlim_cur) {
-            pi.stack_min = (void *)((char *)&obj - (lim.rlim_cur / 4 *
-                                                    3)); /* let 3/4ths of the stack be used only */
+            pi.stack_min = (void *)((char *)&obj - (lim.rlim_cur / 4 * 3)); /* let 3/4ths of the stack be used only */
         } else {
             pi.stack_min = 0; /* indicates not to check stack limit */
         }
@@ -633,7 +622,7 @@ static void saj_parse(VALUE handler, char *json) {
  */
 VALUE
 oj_saj_parse(int argc, VALUE *argv, VALUE self) {
-    char * json  = 0;
+    char  *json  = 0;
     size_t len   = 0;
     VALUE  input = argv[1];
 
@@ -643,7 +632,7 @@ oj_saj_parse(int argc, VALUE *argv, VALUE self) {
     if (rb_type(input) == T_STRING) {
         // the json string gets modified so make a copy of it
         len  = RSTRING_LEN(input) + 1;
-        json = ALLOC_N(char, len);
+        json = OJ_R_ALLOC_N(char, len);
         strcpy(json, StringValuePtr(input));
     } else {
         VALUE          clas = rb_obj_class(input);
@@ -652,7 +641,7 @@ oj_saj_parse(int argc, VALUE *argv, VALUE self) {
         if (oj_stringio_class == clas) {
             s    = rb_funcall2(input, oj_string_id, 0, 0);
             len  = RSTRING_LEN(s) + 1;
-            json = ALLOC_N(char, len);
+            json = OJ_R_ALLOC_N(char, len);
             strcpy(json, rb_string_value_cstr((VALUE *)&s));
 #if !IS_WINDOWS
         } else if (rb_cFile == clas && 0 == FIX2INT(rb_funcall(input, oj_pos_id, 0))) {
@@ -661,7 +650,7 @@ oj_saj_parse(int argc, VALUE *argv, VALUE self) {
 
             len = lseek(fd, 0, SEEK_END);
             lseek(fd, 0, SEEK_SET);
-            json = ALLOC_N(char, len + 1);
+            json = OJ_R_ALLOC_N(char, len + 1);
             if (0 >= (cnt = read(fd, json, len)) || cnt != (ssize_t)len) {
                 rb_raise(rb_eIOError, "failed to read from IO Object.");
             }
@@ -670,14 +659,14 @@ oj_saj_parse(int argc, VALUE *argv, VALUE self) {
         } else if (rb_respond_to(input, oj_read_id)) {
             s    = rb_funcall2(input, oj_read_id, 0, 0);
             len  = RSTRING_LEN(s) + 1;
-            json = ALLOC_N(char, len);
+            json = OJ_R_ALLOC_N(char, len);
             strcpy(json, rb_string_value_cstr((VALUE *)&s));
         } else {
             rb_raise(rb_eArgError, "saj_parse() expected a String or IO Object.");
         }
     }
     saj_parse(*argv, json);
-    xfree(json);
+    OJ_R_FREE(json);
 
     return Qnil;
 }
