@@ -1,8 +1,10 @@
 // Copyright (c) 2021, Peter Ohler, All rights reserved.
 
+#include "mem.h"
 #include "cache.h"
 #include "oj.h"
 #include "parser.h"
+#include "usual.h"
 
 // The Usual delegate builds Ruby objects during parsing. It makes use of
 // three stacks. The first is the value stack. This is where parsed values are
@@ -28,67 +30,12 @@
 
 #define DEBUG 0
 
-// Used to mark the start of each Hash, Array, or Object. The members point at
-// positions of the start in the value stack and if not an Array into the key
-// stack.
-typedef struct _col {
-    long vi;  // value stack index
-    long ki;  // key stack index if an hash else -1 for an array
-} * Col;
-
-typedef union _key {
-    struct {
-        int16_t len;
-        char    buf[30];
-    };
-    struct {
-        int16_t xlen;  // should be the same as len
-        char *  key;
-    };
-} * Key;
-
-#define MISS_AUTO 'A'
-#define MISS_RAISE 'R'
-#define MISS_IGNORE 'I'
-
-typedef struct _delegate {
-    VALUE *vhead;
-    VALUE *vtail;
-    VALUE *vend;
-
-    Col chead;
-    Col ctail;
-    Col cend;
-
-    Key khead;
-    Key ktail;
-    Key kend;
-
-    VALUE (*get_key)(ojParser p, Key kp);
-    struct _cache *key_cache;  // same as str_cache or sym_cache
-    struct _cache *str_cache;
-    struct _cache *sym_cache;
-    struct _cache *class_cache;
-    struct _cache *attr_cache;
-
-    VALUE array_class;
-    VALUE hash_class;
-
-    char *  create_id;
-    uint8_t create_id_len;
-    uint8_t cache_str;
-    uint8_t cache_xrate;
-    uint8_t miss_class;
-    bool    cache_keys;
-    bool    ignore_json_create;
-} * Delegate;
-
 static ID to_f_id = 0;
 static ID ltlt_id = 0;
 static ID hset_id = 0;
 
 static char *str_dup(const char *s, size_t len) {
-    char *d = ALLOC_N(char, len + 1);
+    char *d = OJ_R_ALLOC_N(char, len + 1);
 
     memcpy(d, s, len);
     d[len] = '\0';
@@ -108,7 +55,7 @@ static VALUE form_attr(const char *str, size_t len) {
     char buf[256];
 
     if (sizeof(buf) - 2 <= len) {
-        char *b = ALLOC_N(char, len + 2);
+        char *b = OJ_R_ALLOC_N(char, len + 2);
         ID    id;
 
         *b = '@';
@@ -116,7 +63,7 @@ static VALUE form_attr(const char *str, size_t len) {
         b[len + 1] = '\0';
 
         id = rb_intern3(buf, len + 1, oj_utf8_encoding);
-        xfree(b);
+        OJ_R_FREE(b);
         return id;
     }
     *buf = '@';
@@ -178,27 +125,27 @@ static VALUE form_class_auto(const char *str, size_t len) {
     return resolve_classpath(str, len, true);
 }
 
-static void assure_cstack(Delegate d) {
+static void assure_cstack(Usual d) {
     if (d->cend <= d->ctail + 1) {
         size_t cap = d->cend - d->chead;
         long   pos = d->ctail - d->chead;
 
         cap *= 2;
-        REALLOC_N(d->chead, struct _col, cap);
+        OJ_R_REALLOC_N(d->chead, struct _col, cap);
         d->ctail = d->chead + pos;
         d->cend  = d->chead + cap;
     }
 }
 
 static void push(ojParser p, VALUE v) {
-    Delegate d = (Delegate)p->ctx;
+    Usual d = (Usual)p->ctx;
 
     if (d->vend <= d->vtail) {
         size_t cap = d->vend - d->vhead;
         long   pos = d->vtail - d->vhead;
 
         cap *= 2;
-        REALLOC_N(d->vhead, VALUE, cap);
+        OJ_R_REALLOC_N(d->vhead, VALUE, cap);
         d->vtail = d->vhead + pos;
         d->vend  = d->vhead + cap;
     }
@@ -207,7 +154,7 @@ static void push(ojParser p, VALUE v) {
 }
 
 static VALUE cache_key(ojParser p, Key kp) {
-    Delegate d = (Delegate)p->ctx;
+    Usual d = (Usual)p->ctx;
 
     if ((size_t)kp->len < sizeof(kp->buf)) {
         return cache_intern(d->key_cache, kp->buf, kp->len);
@@ -230,7 +177,7 @@ static VALUE sym_key(ojParser p, Key kp) {
 }
 
 static ID get_attr_id(ojParser p, Key kp) {
-    Delegate d = (Delegate)p->ctx;
+    Usual d = (Usual)p->ctx;
 
     if ((size_t)kp->len < sizeof(kp->buf)) {
         return (ID)cache_intern(d->attr_cache, kp->buf, kp->len);
@@ -239,7 +186,7 @@ static ID get_attr_id(ojParser p, Key kp) {
 }
 
 static void push_key(ojParser p) {
-    Delegate    d    = (Delegate)p->ctx;
+    Usual    d    = (Usual)p->ctx;
     size_t      klen = buf_len(&p->key);
     const char *key  = buf_str(&p->key);
 
@@ -248,7 +195,7 @@ static void push_key(ojParser p) {
         long   pos = d->ktail - d->khead;
 
         cap *= 2;
-        REALLOC_N(d->khead, union _key, cap);
+        OJ_R_REALLOC_N(d->khead, union _key, cap);
         d->ktail = d->khead + pos;
         d->kend  = d->khead + cap;
     }
@@ -263,14 +210,14 @@ static void push_key(ojParser p) {
 }
 
 static void push2(ojParser p, VALUE v) {
-    Delegate d = (Delegate)p->ctx;
+    Usual d = (Usual)p->ctx;
 
     if (d->vend <= d->vtail + 1) {
         size_t cap = d->vend - d->vhead;
         long   pos = d->vtail - d->vhead;
 
         cap *= 2;
-        REALLOC_N(d->vhead, VALUE, cap);
+        OJ_R_REALLOC_N(d->vhead, VALUE, cap);
         d->vtail = d->vhead + pos;
         d->vend  = d->vhead + cap;
     }
@@ -281,7 +228,7 @@ static void push2(ojParser p, VALUE v) {
 }
 
 static void open_object(ojParser p) {
-    Delegate d = (Delegate)p->ctx;
+    Usual d = (Usual)p->ctx;
 
     assure_cstack(d);
     d->ctail->vi = d->vtail - d->vhead;
@@ -291,7 +238,7 @@ static void open_object(ojParser p) {
 }
 
 static void open_object_key(ojParser p) {
-    Delegate d = (Delegate)p->ctx;
+    Usual d = (Usual)p->ctx;
 
     push_key(p);
     assure_cstack(d);
@@ -302,7 +249,7 @@ static void open_object_key(ojParser p) {
 }
 
 static void open_array(ojParser p) {
-    Delegate d = (Delegate)p->ctx;
+    Usual d = (Usual)p->ctx;
 
     assure_cstack(d);
     d->ctail->vi = d->vtail - d->vhead;
@@ -312,7 +259,7 @@ static void open_array(ojParser p) {
 }
 
 static void open_array_key(ojParser p) {
-    Delegate d = (Delegate)p->ctx;
+    Usual d = (Usual)p->ctx;
 
     push_key(p);
     assure_cstack(d);
@@ -324,7 +271,7 @@ static void open_array_key(ojParser p) {
 
 static void close_object(ojParser p) {
     VALUE *  vp;
-    Delegate d = (Delegate)p->ctx;
+    Usual d = (Usual)p->ctx;
 
     d->ctail--;
 
@@ -337,7 +284,7 @@ static void close_object(ojParser p) {
     for (vp = head; kp < d->ktail; kp++, vp += 2) {
         *vp = d->get_key(p, kp);
         if (sizeof(kp->buf) <= (size_t)kp->len) {
-            xfree(kp->key);
+            OJ_R_FREE(kp->key);
         }
     }
     rb_hash_bulk_insert(d->vtail - head, head, obj);
@@ -345,7 +292,7 @@ static void close_object(ojParser p) {
     for (vp = head; kp < d->ktail; kp++, vp += 2) {
         rb_hash_aset(obj, d->get_key(p, kp), *(vp + 1));
         if (sizeof(kp->buf) <= (size_t)kp->len) {
-            xfree(kp->key);
+            OJ_R_FREE(kp->key);
         }
     }
 #endif
@@ -357,7 +304,7 @@ static void close_object(ojParser p) {
 
 static void close_object_class(ojParser p) {
     VALUE *  vp;
-    Delegate d = (Delegate)p->ctx;
+    Usual d = (Usual)p->ctx;
 
     d->ctail--;
 
@@ -369,7 +316,7 @@ static void close_object_class(ojParser p) {
     for (vp = head; kp < d->ktail; kp++, vp += 2) {
         rb_funcall(obj, hset_id, 2, d->get_key(p, kp), *(vp + 1));
         if (sizeof(kp->buf) <= (size_t)kp->len) {
-            xfree(kp->key);
+            OJ_R_FREE(kp->key);
         }
     }
     d->ktail = d->khead + c->ki;
@@ -380,7 +327,7 @@ static void close_object_class(ojParser p) {
 
 static void close_object_create(ojParser p) {
     VALUE *  vp;
-    Delegate d = (Delegate)p->ctx;
+    Usual d = (Usual)p->ctx;
 
     d->ctail--;
 
@@ -397,7 +344,7 @@ static void close_object_create(ojParser p) {
             for (vp = head; kp < d->ktail; kp++, vp += 2) {
                 *vp = d->get_key(p, kp);
                 if (sizeof(kp->buf) <= (size_t)kp->len) {
-                    xfree(kp->key);
+                    OJ_R_FREE(kp->key);
                 }
             }
             rb_hash_bulk_insert(d->vtail - head, head, obj);
@@ -405,7 +352,7 @@ static void close_object_create(ojParser p) {
             for (vp = head; kp < d->ktail; kp++, vp += 2) {
                 rb_hash_aset(obj, d->get_key(p, kp), *(vp + 1));
                 if (sizeof(kp->buf) <= (size_t)kp->len) {
-                    xfree(kp->key);
+                    OJ_R_FREE(kp->key);
                 }
             }
 #endif
@@ -414,7 +361,7 @@ static void close_object_create(ojParser p) {
             for (vp = head; kp < d->ktail; kp++, vp += 2) {
                 rb_funcall(obj, hset_id, 2, d->get_key(p, kp), *(vp + 1));
                 if (sizeof(kp->buf) <= (size_t)kp->len) {
-                    xfree(kp->key);
+                    OJ_R_FREE(kp->key);
                 }
             }
         }
@@ -429,7 +376,7 @@ static void close_object_create(ojParser p) {
             for (vp = head; kp < d->ktail; kp++, vp += 2) {
                 *vp = d->get_key(p, kp);
                 if (sizeof(kp->buf) <= (size_t)kp->len) {
-                    xfree(kp->key);
+                    OJ_R_FREE(kp->key);
                 }
             }
             rb_hash_bulk_insert(d->vtail - head, head, arg);
@@ -437,7 +384,7 @@ static void close_object_create(ojParser p) {
             for (vp = head; kp < d->ktail; kp++, vp += 2) {
                 rb_hash_aset(arg, d->get_key(p, kp), *(vp + 1));
                 if (sizeof(kp->buf) <= (size_t)kp->len) {
-                    xfree(kp->key);
+                    OJ_R_FREE(kp->key);
                 }
             }
 #endif
@@ -447,7 +394,7 @@ static void close_object_create(ojParser p) {
             for (vp = head; kp < d->ktail; kp++, vp += 2) {
                 rb_ivar_set(obj, get_attr_id(p, kp), *(vp + 1));
                 if (sizeof(kp->buf) <= (size_t)kp->len) {
-                    xfree(kp->key);
+                    OJ_R_FREE(kp->key);
                 }
             }
         }
@@ -459,7 +406,7 @@ static void close_object_create(ojParser p) {
 }
 
 static void close_array(ojParser p) {
-    Delegate d = (Delegate)p->ctx;
+    Usual d = (Usual)p->ctx;
 
     d->ctail--;
     VALUE *        head = d->vhead + d->ctail->vi + 1;
@@ -472,7 +419,7 @@ static void close_array(ojParser p) {
 
 static void close_array_class(ojParser p) {
     VALUE *  vp;
-    Delegate d = (Delegate)p->ctx;
+    Usual d = (Usual)p->ctx;
 
     d->ctail--;
     VALUE *        head = d->vhead + d->ctail->vi + 1;
@@ -585,7 +532,7 @@ static void add_big_as_ruby_key(ojParser p) {
 }
 
 static void add_str(ojParser p) {
-    Delegate       d = (Delegate)p->ctx;
+    Usual       d = (Usual)p->ctx;
     volatile VALUE rstr;
     const char *   str = buf_str(&p->buf);
     size_t         len = buf_len(&p->buf);
@@ -599,7 +546,7 @@ static void add_str(ojParser p) {
 }
 
 static void add_str_key(ojParser p) {
-    Delegate       d = (Delegate)p->ctx;
+    Usual       d = (Usual)p->ctx;
     volatile VALUE rstr;
     const char *   str = buf_str(&p->buf);
     size_t         len = buf_len(&p->buf);
@@ -614,7 +561,7 @@ static void add_str_key(ojParser p) {
 }
 
 static void add_str_key_create(ojParser p) {
-    Delegate       d = (Delegate)p->ctx;
+    Usual       d = (Usual)p->ctx;
     volatile VALUE rstr;
     const char *   str  = buf_str(&p->buf);
     size_t         len  = buf_len(&p->buf);
@@ -648,7 +595,7 @@ static void add_str_key_create(ojParser p) {
 }
 
 static VALUE result(ojParser p) {
-    Delegate d = (Delegate)p->ctx;
+    Usual d = (Usual)p->ctx;
 
     if (d->vhead < d->vtail) {
         return *d->vhead;
@@ -657,7 +604,7 @@ static VALUE result(ojParser p) {
 }
 
 static void start(ojParser p) {
-    Delegate d = (Delegate)p->ctx;
+    Usual d = (Usual)p->ctx;
 
     d->vtail = d->vhead;
     d->ctail = d->chead;
@@ -665,7 +612,7 @@ static void start(ojParser p) {
 }
 
 static void dfree(ojParser p) {
-    Delegate d = (Delegate)p->ctx;
+    Usual d = (Usual)p->ctx;
 
     cache_free(d->str_cache);
     cache_free(d->attr_cache);
@@ -675,19 +622,19 @@ static void dfree(ojParser p) {
     if (NULL != d->class_cache) {
         cache_free(d->class_cache);
     }
-    xfree(d->vhead);
-    xfree(d->chead);
-    xfree(d->khead);
-    xfree(d->create_id);
-    xfree(p->ctx);
+    OJ_R_FREE(d->vhead);
+    OJ_R_FREE(d->chead);
+    OJ_R_FREE(d->khead);
+    OJ_R_FREE(d->create_id);
+    OJ_R_FREE(p->ctx);
     p->ctx = NULL;
 }
 
 static void mark(ojParser p) {
-    if (NULL == p->ctx) {
+    if (NULL == p || NULL == p->ctx) {
         return;
     }
-    Delegate d = (Delegate)p->ctx;
+    Usual d = (Usual)p->ctx;
     VALUE *  vp;
 
     if (NULL == d) {
@@ -720,13 +667,13 @@ struct opt {
 };
 
 static VALUE opt_array_class(ojParser p, VALUE value) {
-    Delegate d = (Delegate)p->ctx;
+    Usual d = (Usual)p->ctx;
 
     return d->array_class;
 }
 
 static VALUE opt_array_class_set(ojParser p, VALUE value) {
-    Delegate d = (Delegate)p->ctx;
+    Usual d = (Usual)p->ctx;
 
     if (Qnil == value) {
         p->funcs[TOP_FUN].close_array    = close_array;
@@ -747,13 +694,13 @@ static VALUE opt_array_class_set(ojParser p, VALUE value) {
 }
 
 static VALUE opt_cache_keys(ojParser p, VALUE value) {
-    Delegate d = (Delegate)p->ctx;
+    Usual d = (Usual)p->ctx;
 
     return d->cache_keys ? Qtrue : Qfalse;
 }
 
 static VALUE opt_cache_keys_set(ojParser p, VALUE value) {
-    Delegate d = (Delegate)p->ctx;
+    Usual d = (Usual)p->ctx;
 
     if (Qtrue == value) {
         d->cache_keys = true;
@@ -775,13 +722,13 @@ static VALUE opt_cache_keys_set(ojParser p, VALUE value) {
 }
 
 static VALUE opt_cache_strings(ojParser p, VALUE value) {
-    Delegate d = (Delegate)p->ctx;
+    Usual d = (Usual)p->ctx;
 
     return INT2NUM((int)d->cache_str);
 }
 
 static VALUE opt_cache_strings_set(ojParser p, VALUE value) {
-    Delegate d     = (Delegate)p->ctx;
+    Usual d     = (Usual)p->ctx;
     int      limit = NUM2INT(value);
 
     if (CACHE_MAX_KEY < limit) {
@@ -795,13 +742,13 @@ static VALUE opt_cache_strings_set(ojParser p, VALUE value) {
 }
 
 static VALUE opt_cache_expunge(ojParser p, VALUE value) {
-    Delegate d = (Delegate)p->ctx;
+    Usual d = (Usual)p->ctx;
 
     return INT2NUM((int)d->cache_xrate);
 }
 
 static VALUE opt_cache_expunge_set(ojParser p, VALUE value) {
-    Delegate d    = (Delegate)p->ctx;
+    Usual d    = (Usual)p->ctx;
     int      rate = NUM2INT(value);
 
     if (rate < 0) {
@@ -819,26 +766,26 @@ static VALUE opt_cache_expunge_set(ojParser p, VALUE value) {
 }
 
 static VALUE opt_capacity(ojParser p, VALUE value) {
-    Delegate d = (Delegate)p->ctx;
+    Usual d = (Usual)p->ctx;
 
     return ULONG2NUM(d->vend - d->vhead);
 }
 
 static VALUE opt_capacity_set(ojParser p, VALUE value) {
-    Delegate d   = (Delegate)p->ctx;
+    Usual d   = (Usual)p->ctx;
     long     cap = NUM2LONG(value);
 
     if (d->vend - d->vhead < cap) {
         long pos = d->vtail - d->vhead;
 
-        REALLOC_N(d->vhead, VALUE, cap);
+        OJ_R_REALLOC_N(d->vhead, VALUE, cap);
         d->vtail = d->vhead + pos;
         d->vend  = d->vhead + cap;
     }
     if (d->kend - d->khead < cap) {
         long pos = d->ktail - d->khead;
 
-        REALLOC_N(d->khead, union _key, cap);
+        OJ_R_REALLOC_N(d->khead, union _key, cap);
         d->ktail = d->khead + pos;
         d->kend  = d->khead + cap;
     }
@@ -846,13 +793,13 @@ static VALUE opt_capacity_set(ojParser p, VALUE value) {
 }
 
 static VALUE opt_class_cache(ojParser p, VALUE value) {
-    Delegate d = (Delegate)p->ctx;
+    Usual d = (Usual)p->ctx;
 
     return (NULL != d->class_cache) ? Qtrue : Qfalse;
 }
 
 static VALUE opt_class_cache_set(ojParser p, VALUE value) {
-    Delegate d = (Delegate)p->ctx;
+    Usual d = (Usual)p->ctx;
 
     if (Qtrue == value) {
         if (NULL == d->class_cache) {
@@ -866,7 +813,7 @@ static VALUE opt_class_cache_set(ojParser p, VALUE value) {
 }
 
 static VALUE opt_create_id(ojParser p, VALUE value) {
-    Delegate d = (Delegate)p->ctx;
+    Usual d = (Usual)p->ctx;
 
     if (NULL == d->create_id) {
         return Qnil;
@@ -875,7 +822,7 @@ static VALUE opt_create_id(ojParser p, VALUE value) {
 }
 
 static VALUE opt_create_id_set(ojParser p, VALUE value) {
-    Delegate d = (Delegate)p->ctx;
+    Usual d = (Usual)p->ctx;
 
     if (Qnil == value) {
         d->create_id                 = NULL;
@@ -985,13 +932,13 @@ static VALUE opt_decimal_set(ojParser p, VALUE value) {
 }
 
 static VALUE opt_hash_class(ojParser p, VALUE value) {
-    Delegate d = (Delegate)p->ctx;
+    Usual d = (Usual)p->ctx;
 
     return d->hash_class;
 }
 
 static VALUE opt_hash_class_set(ojParser p, VALUE value) {
-    Delegate d = (Delegate)p->ctx;
+    Usual d = (Usual)p->ctx;
 
     if (Qnil != value) {
         rb_check_type(value, T_CLASS);
@@ -1015,13 +962,13 @@ static VALUE opt_hash_class_set(ojParser p, VALUE value) {
 }
 
 static VALUE opt_ignore_json_create(ojParser p, VALUE value) {
-    Delegate d = (Delegate)p->ctx;
+    Usual d = (Usual)p->ctx;
 
     return d->ignore_json_create ? Qtrue : Qfalse;
 }
 
 static VALUE opt_ignore_json_create_set(ojParser p, VALUE value) {
-    Delegate d = (Delegate)p->ctx;
+    Usual d = (Usual)p->ctx;
 
     d->ignore_json_create = (Qtrue == value);
 
@@ -1029,7 +976,7 @@ static VALUE opt_ignore_json_create_set(ojParser p, VALUE value) {
 }
 
 static VALUE opt_missing_class(ojParser p, VALUE value) {
-    Delegate d = (Delegate)p->ctx;
+    Usual d = (Usual)p->ctx;
 
     switch (d->miss_class) {
     case MISS_AUTO: return ID2SYM(rb_intern("auto"));
@@ -1040,7 +987,7 @@ static VALUE opt_missing_class(ojParser p, VALUE value) {
 }
 
 static VALUE opt_missing_class_set(ojParser p, VALUE value) {
-    Delegate       d = (Delegate)p->ctx;
+    Usual       d = (Usual)p->ctx;
     const char *   mode;
     volatile VALUE s;
 
@@ -1091,13 +1038,13 @@ static VALUE opt_omit_null_set(ojParser p, VALUE value) {
 }
 
 static VALUE opt_symbol_keys(ojParser p, VALUE value) {
-    Delegate d = (Delegate)p->ctx;
+    Usual d = (Usual)p->ctx;
 
     return (NULL != d->sym_cache) ? Qtrue : Qfalse;
 }
 
 static VALUE opt_symbol_keys_set(ojParser p, VALUE value) {
-    Delegate d = (Delegate)p->ctx;
+    Usual d = (Usual)p->ctx;
 
     if (Qtrue == value) {
         d->sym_cache = cache_create(0, form_sym, true, false);
@@ -1121,33 +1068,33 @@ static VALUE opt_symbol_keys_set(ojParser p, VALUE value) {
 static VALUE option(ojParser p, const char *key, VALUE value) {
     struct opt *op;
     struct opt  opts[] = {
-        {.name = "array_class", .func = opt_array_class},
-        {.name = "array_class=", .func = opt_array_class_set},
-        {.name = "cache_keys", .func = opt_cache_keys},
-        {.name = "cache_keys=", .func = opt_cache_keys_set},
-        {.name = "cache_strings", .func = opt_cache_strings},
-        {.name = "cache_strings=", .func = opt_cache_strings_set},
-        {.name = "cache_expunge", .func = opt_cache_expunge},
-        {.name = "cache_expunge=", .func = opt_cache_expunge_set},
-        {.name = "capacity", .func = opt_capacity},
-        {.name = "capacity=", .func = opt_capacity_set},
-        {.name = "class_cache", .func = opt_class_cache},
-        {.name = "class_cache=", .func = opt_class_cache_set},
-        {.name = "create_id", .func = opt_create_id},
-        {.name = "create_id=", .func = opt_create_id_set},
-        {.name = "decimal", .func = opt_decimal},
-        {.name = "decimal=", .func = opt_decimal_set},
-        {.name = "hash_class", .func = opt_hash_class},
-        {.name = "hash_class=", .func = opt_hash_class_set},
-        {.name = "ignore_json_create", .func = opt_ignore_json_create},
-        {.name = "ignore_json_create=", .func = opt_ignore_json_create_set},
-        {.name = "missing_class", .func = opt_missing_class},
-        {.name = "missing_class=", .func = opt_missing_class_set},
-        {.name = "omit_null", .func = opt_omit_null},
-        {.name = "omit_null=", .func = opt_omit_null_set},
-        {.name = "symbol_keys", .func = opt_symbol_keys},
-        {.name = "symbol_keys=", .func = opt_symbol_keys_set},
-        {.name = NULL},
+         {.name = "array_class", .func = opt_array_class},
+         {.name = "array_class=", .func = opt_array_class_set},
+         {.name = "cache_keys", .func = opt_cache_keys},
+         {.name = "cache_keys=", .func = opt_cache_keys_set},
+         {.name = "cache_strings", .func = opt_cache_strings},
+         {.name = "cache_strings=", .func = opt_cache_strings_set},
+         {.name = "cache_expunge", .func = opt_cache_expunge},
+         {.name = "cache_expunge=", .func = opt_cache_expunge_set},
+         {.name = "capacity", .func = opt_capacity},
+         {.name = "capacity=", .func = opt_capacity_set},
+         {.name = "class_cache", .func = opt_class_cache},
+         {.name = "class_cache=", .func = opt_class_cache_set},
+         {.name = "create_id", .func = opt_create_id},
+         {.name = "create_id=", .func = opt_create_id_set},
+         {.name = "decimal", .func = opt_decimal},
+         {.name = "decimal=", .func = opt_decimal_set},
+         {.name = "hash_class", .func = opt_hash_class},
+         {.name = "hash_class=", .func = opt_hash_class_set},
+         {.name = "ignore_json_create", .func = opt_ignore_json_create},
+         {.name = "ignore_json_create=", .func = opt_ignore_json_create_set},
+         {.name = "missing_class", .func = opt_missing_class},
+         {.name = "missing_class=", .func = opt_missing_class_set},
+         {.name = "omit_null", .func = opt_omit_null},
+         {.name = "omit_null=", .func = opt_omit_null_set},
+         {.name = "symbol_keys", .func = opt_symbol_keys},
+         {.name = "symbol_keys=", .func = opt_symbol_keys_set},
+         {.name = NULL},
     };
 
     for (op = opts; NULL != op->name; op++) {
@@ -1162,20 +1109,19 @@ static VALUE option(ojParser p, const char *key, VALUE value) {
 
 ///// the set up //////////////////////////////////////////////////////////////
 
-void oj_set_parser_usual(ojParser p) {
-    Delegate d   = ALLOC(struct _delegate);
+void oj_init_usual(ojParser p, Usual d) {
     int      cap = 4096;
 
-    d->vhead = ALLOC_N(VALUE, cap);
+    d->vhead = OJ_R_ALLOC_N(VALUE, cap);
     d->vend  = d->vhead + cap;
     d->vtail = d->vhead;
 
-    d->khead = ALLOC_N(union _key, cap);
+    d->khead = OJ_R_ALLOC_N(union _key, cap);
     d->kend  = d->khead + cap;
     d->ktail = d->khead;
 
     cap      = 256;
-    d->chead = ALLOC_N(struct _col, cap);
+    d->chead = OJ_R_ALLOC_N(struct _col, cap);
     d->cend  = d->chead + cap;
     d->ctail = d->chead;
 
@@ -1235,6 +1181,8 @@ void oj_set_parser_usual(ojParser p) {
     d->class_cache = NULL;
     d->key_cache   = d->str_cache;
 
+    // The parser fields are set but the functions can be replaced by a
+    // delegate that wraps the usual delegate.
     p->ctx    = (void *)d;
     p->option = option;
     p->result = result;
@@ -1252,3 +1200,9 @@ void oj_set_parser_usual(ojParser p) {
         hset_id = rb_intern("[]=");
     }
 }
+
+void oj_set_parser_usual(ojParser p) {
+    Usual d = OJ_R_ALLOC(struct _usual);
+
+    oj_init_usual(p, d);
+}
