@@ -221,13 +221,6 @@ class ObjectJuice < Minitest::Test
 
   def teardown
     Oj.default_options = @default_options
-#=begin
-    if '3.1.0' <= RUBY_VERSION && !(RbConfig::CONFIG['host_os'] =~ /(mingw|mswin)/)
-      #Oj::debug_odd("teardown before GC.verify_compaction_references")
-      verify_gc_compaction
-      #Oj::debug_odd("teardown after GC.verify_compaction_references")
-    end
-#=end
   end
 
   def test_nil
@@ -828,10 +821,10 @@ class ObjectJuice < Minitest::Test
   def test_range_object
     Oj.default_options = { :mode => :object }
     json = Oj.dump(1..7, :mode => :object, :indent => 0)
-    if 'rubinius' == $ruby
-      assert(%{{"^O":"Range","begin":1,"end":7,"exclude_end?":false}} == json)
-    else
+    if 'ruby' == $ruby
       assert_equal(%{{"^u":["Range",1,7,false]}}, json)
+    else
+      assert(%{{"^O":"Range","begin":1,"end":7,"exclude_end?":false}} == json)
     end
     dump_and_load(1..7, false)
     dump_and_load(1..1, false)
