@@ -559,9 +559,6 @@ class Juice < Minitest::Test
   end
 
   def test_io_file
-    # Windows does not support fork
-    return if RbConfig::CONFIG['host_os'] =~ /(mingw|mswin)/
-
     src = { 'x' => true, 'y' => 58, 'z' => [1, 2, 3]}
     filename = File.join(File.dirname(__FILE__), 'open_file_test.json')
     File.open(filename, "w") { |f|
@@ -574,6 +571,8 @@ class Juice < Minitest::Test
   end
 
   def test_io_stream
+    skip 'needs fork' unless Process.respond_to?(:fork)
+
     IO.pipe do |r, w|
       if fork
 	r.close
