@@ -18,8 +18,16 @@ require 'test_object'
 require 'test_saj'
 require 'test_scp'
 require 'test_strict'
-require 'test_various'
 require 'test_rails'
 require 'test_wab'
 require 'test_writer'
 require 'test_integer_range'
+
+at_exit do
+  require 'helper'
+  if '3.1.0' <= RUBY_VERSION && !(RbConfig::CONFIG['host_os'] =~ /(mingw|mswin)/)
+    #Oj::debug_odd("teardown before GC.verify_compaction_references")
+    verify_gc_compaction
+    #Oj::debug_odd("teardown after GC.verify_compaction_references")
+  end
+end
